package nl.tue.id.tim;

import java.util.*;
import java.util.regex.*;

import com.valhalla.jbother.*;
import com.valhalla.jbother.jabber.*;
import com.valhalla.jbother.plugins.events.*;
import com.valhalla.pluginmanager.*;
import javax.swing.text.*;
import com.valhalla.Logger;
import javax.swing.text.html.*;
import java.io.*;

/**
 * <p>TIM Emoticon Collector plugin for JBother.</p>
 *
 * <p>Description: This plugin collects emoticons from the conversations.</p>
 *
 * <p>Copyright: Copyright (c) 2005 Jun Hu</p>
 *
 * <p>Company: ID, TU/e</p>
 *
 * @author Jun Hu
 * @version 1.0
 */
public class EmoticonCollector
    implements Plugin, PluginEventListener {

  /**
   * i18n translation resources.
   */
  private ResourceBundle resources = JBother.resources;

  private EmoticonFrame frame = new EmoticonFrame();

  String regex = ":\\)|:-\\)|" +
      ":\\(|:-\\(|"+
      ":\\]|:-\\]|"+
      ":P|:p|"+
      ":o|:O|"+
      "8\\)|B\\)|8-\\)|B-\\)";

  public EmoticonCollector() {
  }

  /**
   * Initiate the plugin.
   * <br> add this plugin to the PluginChain
   *
   * @return boolean. if the plugin is sucessfully loaded, retun true.
   */
  public boolean init() {
    PluginChain.addListener(this);
    frame.pack();
    frame.show();
    return true;
  }

  /**
   * Unloads the plugin. remove this plugin from the PluginChain.
   */
  public void unload() {
    PluginChain.removeListener(this);
    frame.hide();
  }

  /**
   * This is the event handling interface defined in JBother Plugin.
   * Emoticon Collector monitors the conversation events distributed from ChatPanel.
   *
   * @param pluginEvent PluginEvent
   */
  public void handleEvent(PluginEvent pluginEvent) {
    if (pluginEvent instanceof ConversationEvent) {
      conversationHandler( (ConversationEvent) pluginEvent);
    }
  }

  /**
   * conversationHandler
   * @param pluginEvent ConversationEvent
   */
  private void conversationHandler(ConversationEvent pluginEvent) {
    ChatPanel conversation = null;
    if (pluginEvent.getSource() instanceof ChatPanel) {
      conversation = (ChatPanel) pluginEvent.getSource();
    }
    if (conversation == null) {
      return;
    }
    BuddyStatus buddy = conversation.getBuddy();
    String content = pluginEvent.getContent();
    Pattern p = Pattern.compile(regex);
      Matcher m = p.matcher(content);

    if (pluginEvent.getType() == ConversationEvent.Type.RECEIVEING_MESSAGE) {
      while (m.find()) {
        Logger.debug(m.group());
        String icon = Emoticons.getInstance().replaceIcons(" " + m.group() +
            " ");
        try {
          HTMLEditorKit kit = (HTMLEditorKit) frame.getBuddyEditorPane().
              getEditorKit();
          HTMLDocument doc = (HTMLDocument) frame.getBuddyEditorPane().
              getDocument();
          kit.insertHTML(doc, doc.getLength(), icon, 0, 0, null);
        }
        catch (IOException ex1) {
        }
        catch (BadLocationException ex1) {
        }
      }
    }
    else if (pluginEvent.getType() == ConversationEvent.Type.SENDING_MESSAGE) {
      while (m.find()) {
        Logger.debug(m.group());
        String icon = Emoticons.getInstance().replaceIcons(" " + m.group() +
            " ");
        try {
          HTMLEditorKit kit = (HTMLEditorKit) frame.getMyEditorPane().
              getEditorKit();
          HTMLDocument doc = (HTMLDocument) frame.getMyEditorPane().
              getDocument();
          kit.insertHTML(doc, doc.getLength(), icon, 0, 0, null);
        }
        catch (IOException ex1) {
        }
        catch (BadLocationException ex1) {
        }
      }
    }
    else if (pluginEvent.getType() == ConversationEvent.Type.INIT) {
    }
  }

}
