package nl.tue.id.tim;

import java.util.*;

import java.awt.*;
import java.awt.event.*;
import javax.swing.*;

import com.valhalla.gui.*;
import com.valhalla.jbother.*;

/**
 * <p>File Transfer Button to be shown on chat panels.</p>
 *
 * <p>Description: A image icon button to be added to and removed from a chat
 * panel. It also uses static members to remember the chat panels that have such
 * a button to unload and reload these buttons.</p>
 *
 * <p>Copyright: Copyright (c) 2005 Jun Hu.</p>
 *
 * <p>Company: ID, TU/e.</p>
 *
 * @author Jun Hu
 * @version 1.0
 */
class FileTransferButton
    extends JButton implements ActionListener {
  /**
   * collection of all the FileTransferButton instances.
   */
  static Vector buttons = new Vector();
  /**
   * the ChatPanel this button to be added to.
   */
  ChatPanel chat;
  /**
   * FileTransfer that created this button. it is needed for call
   * back its upload() function.
   */
  FileTransfer fileTransfer;
  /**
   * FileTransferButton.
   *
   * @param chat ChatPanel
   * @param fileTransfer FileTransfer
   */
  public FileTransferButton(ChatPanel chat, FileTransfer fileTransfer) {
    this.chat = chat;
    this.fileTransfer = fileTransfer;
    this.setIcon(Standard.getIcon(
        "images/buttons/filetrans.gif"));
    chat.getButtonPanel().add(this);
    setPreferredSize(new Dimension(26, 26));
    setToolTipText("Send a file to this buddy");
    addActionListener(this);
    buttons.add(this);
  }

  /**
   * When this button is pressed.
   *
   * @param e ActionEvent
   */
  public void actionPerformed(ActionEvent e) {
    String url = fileTransfer.uploadFile();
    if (url != null) {
      chat.directSend("/file " + url, true);
    }
  }

  /**
   * remove all FileTransferButton instances from its chat panel.
   */
  public static void removeButtons() {
    Iterator b = buttons.iterator();
    while (b.hasNext()) {
      FileTransferButton button = (FileTransferButton) b.next();
      button.chat.getButtonPanel().remove(button);
      button.chat.revalidate();
    }
  }

  /**
   * reload all FileTransferButton instances to its chat panel.
   */
  public static void reloadButtons() {
    Iterator b = buttons.iterator();
    while (b.hasNext()) {
      FileTransferButton button = (FileTransferButton) b.next();
      button.chat.getButtonPanel().add(button);
      button.chat.revalidate();
    }
  }

}
