package nl.tue.id.tim;

import java.util.*;

import java.awt.*;
import java.awt.event.*;
import javax.swing.*;

import org.jivesoftware.smack.packet.*;
import com.valhalla.jbother.*;
import com.valhalla.settings.*;

/**
 * <p>A graphical control panel for the presence watcher.</p>
 *
 * <p>Description: a graphical control panel that has several toggle buttons,
 * each represents a presence state of the user. by pussing the buttons the user
 * may change his/her presence state. When the presence state of the user
 * changes, the control panel will toggle the buttons to reflect the change.
 * </p>
 *
 * <p>Copyright: Copyright (c) 2005 Jun Hu</p>
 *
 * <p>Company: ID, TU/e</p>
 *
 * @author Jun Hu
 * @version 1.0
 */
public class ControlPanel
    extends JFrame {

  private ResourceBundle resources = JBother.resources;
  FlowLayout flowLayout1 = new FlowLayout();
  JToggleButton available = new JToggleButton();
  JToggleButton away = new JToggleButton();
  JToggleButton offline = new JToggleButton();
  JToggleButton chat = new JToggleButton();
  JToggleButton dnd = new JToggleButton();
  JToggleButton xa = new JToggleButton();
  JToggleButton invisible = new JToggleButton();

  PresenceWatcher pw = null;
  public ControlPanel(PresenceWatcher pw) {
    this.pw = pw;
    try {
      jbInit();
    }
    catch (Exception ex) {
      ex.printStackTrace();
    }
    pack();
    setPreferredLocation();
    setPreferredDimension();
  }

  private void jbInit() throws Exception {
    this.getContentPane().setLayout(flowLayout1);
    offline.setText(resources.getString("offline"));
    offline.addActionListener(new ControlPanel_offline_actionAdapter(this));
    available.setText(resources.getString("available"));
    available.addActionListener(new ControlPanel_available_actionAdapter(this));
    away.setText(resources.getString("away"));
    away.addActionListener(new ControlPanel_away_actionAdapter(this));
    chat.setText(resources.getString("chat"));
    chat.addActionListener(new ControlPanel_chat_actionAdapter(this));
    dnd.setText(resources.getString("dnd"));
    dnd.addActionListener(new ControlPanel_dnd_actionAdapter(this));
    xa.setText(resources.getString("xa"));
    xa.addActionListener(new ControlPanel_xa_actionAdapter(this));
    invisible.setText(resources.getString("invisible"));
    invisible.addActionListener(new ControlPanel_invisible_actionAdapter(this));

    offline.setIcon(StatusIconCache.getStatusIcon(null));
    offline.setSelected(true);
    available.setIcon(StatusIconCache.getStatusIcon(Presence.Mode.AVAILABLE));
    away.setIcon(StatusIconCache.getStatusIcon(Presence.Mode.AWAY));
    chat.setIcon(StatusIconCache.getStatusIcon(Presence.Mode.CHAT));
    dnd.setIcon(StatusIconCache.getStatusIcon(Presence.Mode.DO_NOT_DISTURB));
    xa.setIcon(StatusIconCache.getStatusIcon(Presence.Mode.EXTENDED_AWAY));
    invisible.setIcon(StatusIconCache.getStatusIcon(Presence.Mode.INVISIBLE));

    offline.setSelected(true);

    this.setIconImage(StatusIconCache.getStatusIcon(null).getImage());
    this.setTitle("Presence Watcher");
    this.getContentPane().add(offline);
    this.getContentPane().add(available);
    this.getContentPane().add(away);
    this.getContentPane().add(chat);
    this.getContentPane().add(dnd);
    this.getContentPane().add(xa);
    this.getContentPane().add(invisible);
  }

  /**
   * Loads the saved position settings from any previous settings
   */
  protected void setPreferredLocation() {
    //load the settings from the settings file
    String xString = Settings.getInstance().getProperty("presenceWatcherX");
    String yString = Settings.getInstance().getProperty("presenceWatcherY");

    if (yString == null) {
      yString = "100";
    }
    if (xString == null) {
      xString = "100";
    }

    double x = Double.parseDouble(xString);
    double y = Double.parseDouble(yString);

    if (x < 0.0) {
      x = 100.0;
    }
    if (y < 0.0) {
      y = 100.0;
    }

    setLocation( (int) x, (int) y);
  }

  /**
   * Loads the saved dimenstion settings from any previous settings
   */

  protected void setPreferredDimension() {
    //load the settings from the settings file
    String xString = Settings.getInstance().getProperty("presenceWatcherWidth");
    String yString = Settings.getInstance().getProperty("presenceWatcherHeight");

    if (yString == null) {
      yString = "-1";
    }
    if (xString == null) {
      xString = "-1";
    }

    double x = Double.parseDouble(xString);
    double y = Double.parseDouble(yString);

    if (x < 0.0) {
      return;
    }
    if (y < 0.0) {
      return;
    }

    setSize( (int) x, (int) y);
  }

  /**
   * Saves the current settings - like the height and width of the control panel
   */
  public void saveSettings() {
    if (isVisible()) {
      //save the current buddy list size and position
      Dimension size = new Dimension(getSize());
      Point location = new Point(getLocationOnScreen());
      Settings.getInstance().setProperty("presenceWatcherX",
                                         new Double(location.getX()).toString());
      Settings.getInstance().setProperty("presenceWatcherY",
                                         new Double(location.getY()).toString());
      Settings.getInstance().setProperty("presenceWatcherWidth",
                                         new Double(size.width).toString());
      Settings.getInstance().setProperty("presenceWatcherHeight",
                                         new Double(size.height).toString());
    }

    Settings.getInstance().writeSettings();
  }

  /**
   * update the button state to reflect the latest status of the user's presence
   * mode.
   *
   * @param mode Mode
   */
  public void currentPresenceMode(Presence.Mode mode) {

    if (mode == Presence.Mode.AVAILABLE) {
      deSelectAll(this.available);
    }
    else if (mode == Presence.Mode.AWAY) {
      deSelectAll(this.away);
    }
    else if (mode == Presence.Mode.CHAT) {
      deSelectAll(this.chat);
    }
    else if (mode == Presence.Mode.DO_NOT_DISTURB) {
      deSelectAll(this.dnd);
    }
    else if (mode == Presence.Mode.EXTENDED_AWAY) {
      deSelectAll(this.xa);
    }
    else if (mode == Presence.Mode.INVISIBLE) {
      deSelectAll(this.invisible);
    }
    else {
      deSelectAll(offline);
    }
  }

  /**
   * deselect all the buttons except the one specifiled in the param.
   *
   * @param except JToggleButton
   */
  public void deSelectAll(JToggleButton except) {
    if (except != offline) {
      offline.setSelected(false);
      offline.setEnabled(true);
    }
    if (except != available) {
      available.setSelected(false);
      available.setEnabled(true);
    }
    if (except != chat) {
      chat.setSelected(false);
      chat.setEnabled(true);
    }
    if (except != away) {
      away.setSelected(false);
      away.setEnabled(true);
    }
    if (except != xa) {
      xa.setSelected(false);
      xa.setEnabled(true);
    }
    if (except != dnd) {
      dnd.setSelected(false);
      dnd.setEnabled(true);
    }
    if (except != invisible) {
      invisible.setSelected(false);
      invisible.setEnabled(true);
    }
    except.setEnabled(false);
    this.setIconImage( ( (ImageIcon) except.getIcon()).getImage());
  }

  public void offline_actionPerformed(ActionEvent e) {
    if (offline.isSelected()) {
      deSelectAll(offline);
      if (BuddyList.getInstance().getCurrentPresenceMode() != null) {
        BuddyList.getInstance().signOff();
      }
    }
  }

  public void available_actionPerformed(ActionEvent e) {
    if (available.isSelected()) {
      deSelectAll(available);
      BuddyList.getInstance().setStatus(Presence.Mode.AVAILABLE,
                                        resources.getString("available"), false);
    }
  }

  public void away_actionPerformed(ActionEvent e) {
    if (away.isSelected()) {
      deSelectAll(away);
      BuddyList.getInstance().setStatus(Presence.Mode.AWAY,
                                        resources.getString("away"), false);
    }
  }

  public void chat_actionPerformed(ActionEvent e) {
    if (chat.isSelected()) {
      deSelectAll(chat);
      BuddyList.getInstance().setStatus(Presence.Mode.CHAT,
                                        resources.getString("chat"), false);
    }
  }

  public void dnd_actionPerformed(ActionEvent e) {
    if (dnd.isSelected()) {
      deSelectAll(dnd);
      BuddyList.getInstance().setStatus(Presence.Mode.DO_NOT_DISTURB,
                                        resources.getString("dnd"), false);
    }
  }

  public void xa_actionPerformed(ActionEvent e) {
    if (xa.isSelected()) {
      deSelectAll(xa);
      BuddyList.getInstance().setStatus(Presence.Mode.EXTENDED_AWAY,
                                        resources.getString("xa"), false);
    }
  }

  public void invisible_actionPerformed(ActionEvent e) {
    if (invisible.isSelected()) {
      deSelectAll(invisible);
      BuddyList.getInstance().setStatus(Presence.Mode.INVISIBLE,
                                        resources.getString("invisible"), false);
    }
  }

}

class ControlPanel_invisible_actionAdapter
    implements ActionListener {
  private ControlPanel adaptee;
  ControlPanel_invisible_actionAdapter(ControlPanel adaptee) {
    this.adaptee = adaptee;
  }

  public void actionPerformed(ActionEvent e) {
    adaptee.invisible_actionPerformed(e);
  }
}

class ControlPanel_xa_actionAdapter
    implements ActionListener {
  private ControlPanel adaptee;
  ControlPanel_xa_actionAdapter(ControlPanel adaptee) {
    this.adaptee = adaptee;
  }

  public void actionPerformed(ActionEvent e) {
    adaptee.xa_actionPerformed(e);
  }
}

class ControlPanel_dnd_actionAdapter
    implements ActionListener {
  private ControlPanel adaptee;
  ControlPanel_dnd_actionAdapter(ControlPanel adaptee) {
    this.adaptee = adaptee;
  }

  public void actionPerformed(ActionEvent e) {
    adaptee.dnd_actionPerformed(e);
  }
}

class ControlPanel_chat_actionAdapter
    implements ActionListener {
  private ControlPanel adaptee;
  ControlPanel_chat_actionAdapter(ControlPanel adaptee) {
    this.adaptee = adaptee;
  }

  public void actionPerformed(ActionEvent e) {
    adaptee.chat_actionPerformed(e);
  }
}

class ControlPanel_away_actionAdapter
    implements ActionListener {
  private ControlPanel adaptee;
  ControlPanel_away_actionAdapter(ControlPanel adaptee) {
    this.adaptee = adaptee;
  }

  public void actionPerformed(ActionEvent e) {
    adaptee.away_actionPerformed(e);
  }
}

class ControlPanel_available_actionAdapter
    implements ActionListener {
  private ControlPanel adaptee;
  ControlPanel_available_actionAdapter(ControlPanel adaptee) {
    this.adaptee = adaptee;
  }

  public void actionPerformed(ActionEvent e) {
    adaptee.available_actionPerformed(e);
  }
}

class ControlPanel_offline_actionAdapter
    implements ActionListener {
  private ControlPanel adaptee;
  ControlPanel_offline_actionAdapter(ControlPanel adaptee) {
    this.adaptee = adaptee;
  }

  public void actionPerformed(ActionEvent e) {
    adaptee.offline_actionPerformed(e);
  }
}
