package nl.tue.id.tim;

import java.util.*;

import org.jivesoftware.smack.packet.*;
import com.valhalla.*;
import com.valhalla.jbother.*;
import com.valhalla.jbother.jabber.*;
import com.valhalla.jbother.plugins.events.*;
import com.valhalla.pluginmanager.*;

/**
 * <p>TIM Presence Watcher.</p>
 *
 * <p>Description: a plugin for JBother that watches and changes the status
 * change of the user and the buddies. </p>
 *
 * <p>Copyright: Copyright (c) 2005 Jun Hu</p>
 *
 * <p>Company: ID, TU/e</p>
 *
 * @author Jun Hu
 * @version 1.0
 */
public class PresenceWatcher
    implements Plugin, PluginEventListener {

  /**
   * i18n translation resources from JBother
   */
  private ResourceBundle resources = JBother.resources;

  /**
   * a GUI control panel to show and change the stadus of the user and the
   * buddies.
   */
  ControlPanel cp = null;

  public PresenceWatcher() {
    cp = new ControlPanel(this);
  }

  /**
   * when load the plugin, show the control panel and register this plugin to
   * the plugin event chain.
   *
   * @return boolean
   */
  public boolean init() {
    cp.show();
    Presence.Mode mode = BuddyList.getInstance().getCurrentPresenceMode();
    cp.currentPresenceMode(mode);
    PluginChain.addListener(this);

    return true;
  }


  /**
   * Unloads the plugin. remove it from the plugin event chain. hide the control
   * panel.
   */
  public void unload() {
    PluginChain.removeListener(this);
    cp.hide();
  }


  /**
   * this plugin handles three events: the presence change of the user, the
   * presence change of a buddy, and exisiting.
   *
   * @param pluginEvent PluginEvent
   */
  public void handleEvent(PluginEvent pluginEvent) {
    if (pluginEvent instanceof CurrentPresenceModeEvent) {
      currentPresenceModeHandler( (CurrentPresenceModeEvent) pluginEvent);
    }
    else if (pluginEvent instanceof BuddyPresenceModeEvent) {
      buddyPresenceModeHandler( (BuddyPresenceModeEvent) pluginEvent);
    }
    else if (pluginEvent instanceof ExitingEvent) {
      exitingHandler( (ExitingEvent) pluginEvent);
    }
  }

  /**
   * when existing, let the control panel save its screen positions and
   * dimenstions.
   *
   * @param event ExitingEvent
   */
  private void exitingHandler(ExitingEvent event) {
    cp.saveSettings();
  }

  /**
   * buddyPresenceModeHandler
   *
   * @param pluginEvent BuddyPresenceModeEvent
   */
  private void buddyPresenceModeHandler(BuddyPresenceModeEvent pluginEvent) {
    Object source = pluginEvent.getSource();
    BuddyStatus buddy = null;

    if (source instanceof BuddyStatus) {
      buddy = (BuddyStatus) source;
    }

    if (buddy == null) {
      return;
    }

    Presence.Mode mode = buddy.getPresence(buddy.getHighestResource());

    Logger.debug(buddy.getUser() + " a.k.a. " + buddy.getName() + " : " +
                 getStatus(mode));

  }

  /**
   * currentPresenceModeHandler
   *
   * @param pluginEvent CurrentPresenceModeEvent
   */
  private void currentPresenceModeHandler(CurrentPresenceModeEvent pluginEvent) {
    Presence.Mode mode = BuddyList.getInstance().getCurrentPresenceMode();
    Logger.debug("Me: " + getStatus(mode));

    cp.currentPresenceMode(mode);
  }

  /**
   * return description of a certain presence mode.
   *
   * @param mode Mode
   * @return String
   */
  private String getStatus(Presence.Mode mode) {
    String status = resources.getString("offline");

    if (mode == Presence.Mode.AVAILABLE) {
      status = resources.getString("available");
    }
    else if (mode == Presence.Mode.AWAY) {
      status = resources.getString("away");
    }
    else if (mode == Presence.Mode.CHAT) {
      status = resources.getString("chat");
    }
    else if (mode == Presence.Mode.DO_NOT_DISTURB) {
      status = resources.getString("dnd");
    }
    else if (mode == Presence.Mode.EXTENDED_AWAY) {
      status = resources.getString("xa");
    }
    else if (mode == Presence.Mode.INVISIBLE) {
      status = resources.getString("invisible");
    }
    return status;

  }

}
