/*
    Copyright (C) 2003 Adam Olsen

	This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 1, or (at your option)
    any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/

package com.valhalla;

import java.text.SimpleDateFormat;
import java.util.Date;
import java.io.*;

/**
 * Class used for logging/debugging
 *
 * @author Adam Olsen
 * @version 1.0
**/
public class Logger
{
	private static PrintWriter logOut = null;

	/**
	 * Sets the log file.  If no log file is set, it will not be written to
	 * @param file the log file location
	*/
	public static void setLogFile( String file )
	{
		try {
			File logFile = new File( file );
			logOut = new PrintWriter( new FileOutputStream( logFile ), true );
		}
		catch( IOException ex )
		{
			System.out.println( "Could not write to log file" );
		}
	}

	/**
	 * Logs an exceptions stack trace if the log file is open
	 * @param ex the exception
	*/
	public static void logException( Exception ex )
	{
		if( System.getProperty( "debug" ) != null )
		{
			ex.printStackTrace();
		}

		if( logOut != null )
		{
			try {
				ex.printStackTrace( logOut );
			}
			catch( Exception e ) { }
		}
	}

	/**
	 * Closes the log file if it's open
	*/
	public static void closeLog()
	{
		if( logOut != null )
		{
			try {
				logOut.close();
			}
			catch( Exception ex ) { }
		}
	}

	/**
	 * Outputs to the console only if the "debug" system property is set.
	 *
	 * @param message The message to output to the console
	**/
	public static void debug( String message )
	{
		if( System.getProperty( "debug" ) != null )
		{
			com.valhalla.Logger.write( message );
		}

		if( logOut != null )
		{
			SimpleDateFormat formatter = new SimpleDateFormat( "[yyyy-MM-dd @ HH:mm:ss]: " );
			String date = formatter.format( new Date() );

			try {
				logOut.println( date + message );
			}
			catch( Exception ex ) { }
		}
	}

	/**
	 * Output to the console - include the date and time
	 *
	 * @param message The message to output to the console
	**/
	public static void write( String message )
	{
		// get the current date/time and output it prettily to the console
		SimpleDateFormat formatter = new SimpleDateFormat( "[yyyy-MM-dd @ HH:mm:ss]: " );
		String date = formatter.format( new Date() );
		System.out.println( date + message );

	}
}
