/*
	Copyright (C) 2003 Adam Olsen

	This program is free software; you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation; either version 1, or (at your option)
	any later version.

	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with this program; if not, write to the Free Software
	Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/

package com.valhalla.jbother;

import java.awt.*;
import java.awt.event.*;
import java.util.*;
import java.text.*;

import javax.swing.*;
import java.io.*;

import com.valhalla.gui.*;
import com.valhalla.jbother.*;


/**
 * This obiously displays an about dialog with the credits for JBother
 *
 * @author Adam Olsen
 * @version 1.0
 **/
public class AboutDialog extends JDialog
{
	private ResourceBundle resources = ResourceBundle.getBundle( "JBotherBundle" );
	private JButton okButton = new JButton( resources.getString( "okButton" ) );
	private JButton creditsButton = new JButton( resources.getString( "creditsButton" ) );
	private JTextPane textPane = new JTextPane();
	private JScrollPane scrollPane = new JScrollPane( textPane );
	private JPanel middlePanel = new JPanel( new BorderLayout() );
	private JPanel version;
	private JLabel imageLabel = new JLabel( Standard.getIcon( "images/splashimage.png" ) );
	private JPanel buttonPanel = new JPanel();
	private JPanel mainPanel;
	private JPanel container = new JPanel( new BorderLayout() );
	private boolean credits = false;
	private AboutDialog thisPointer = this;

	/**
	 * Sets up the Visual components
	*/
	public AboutDialog()
	{
		super( (Frame)null, "About JBother", false );
		setTitle( resources.getString( "aboutDialogTitle" ) );

		mainPanel = (JPanel)getContentPane();
		container.setLayout( new BorderLayout() );
		container.setBorder( BorderFactory.createEmptyBorder( 0, 15, 15, 15 ) );
		mainPanel.setBorder( BorderFactory.createTitledBorder( "About JBother" ) );

		buttonPanel.setLayout( new BoxLayout( buttonPanel, BoxLayout.X_AXIS ) );
		buttonPanel.add( Box.createHorizontalGlue() );
		buttonPanel.setBorder( BorderFactory.createEmptyBorder( 5, 5, 5, 5 ) );
		buttonPanel.add( okButton );
		buttonPanel.add( creditsButton );
		buttonPanel.add( Box.createHorizontalGlue() );

		createVersionPanel();
		middlePanel.add( version, BorderLayout.NORTH );

		container.add( imageLabel, BorderLayout.NORTH );
		container.add( middlePanel, BorderLayout.CENTER );
		mainPanel.add( buttonPanel, BorderLayout.SOUTH );

		mainPanel.add( container, BorderLayout.CENTER );

		imageLabel.setBorder( BorderFactory.createEmptyBorder( 5, 5, 10, 5 ) );
		textPane.setEditable( false );

		// load in the application credits
		InputStream file = getClass().getClassLoader().getResourceAsStream( "credits.txt" );
		InputStreamReader in = new InputStreamReader( file );
		BufferedReader reader = new BufferedReader( in );
		StringBuffer buffer = new StringBuffer();
		String line;

		try	{

			while( ( line = reader.readLine() ) != null )
			{
				buffer.append( line + "\n" );
			}

			in.close();
		}
		catch( IOException e ) { com.valhalla.Logger.debug( "Couldn't fread credits file." ); }

		textPane.setFont( new Font( "Monospaced", Font.PLAIN, 12 ) );
		textPane.setText( buffer.toString() );
		textPane.setCaretPosition( 0 );

		final AboutDialog thisPointer = this;

		okButton.addActionListener( new ActionListener()
		{
			public void actionPerformed( ActionEvent e )
			{
				DialogTracker.removeDialog( thisPointer );
			}
		} );

		creditsButton.addActionListener( new ActionListener()
		{
			public void actionPerformed( ActionEvent e )
			{
				changeHandler();
			}
		} );

		pack();
		setLocationRelativeTo( null );
		setResizable( false );

		DialogTracker.addDialog( this, true, true );
	}

	/**
	 * Toggles back and forth between showing the logo and version information and
	 * showing the application credits
	*/
	private void changeHandler()
	{
		if( !credits )
		{
			mainPanel.remove( container );
			mainPanel.add( scrollPane );

			creditsButton.setText( resources.getString( "info" ) );
			mainPanel.setBorder( BorderFactory.createTitledBorder( "JBother Credits" ) );

			credits = true;
		}
		else {
			mainPanel.remove( scrollPane );
			mainPanel.add( container );

			creditsButton.setText( resources.getString( "creditsButton" ) );
			mainPanel.setBorder( BorderFactory.createTitledBorder( "About JBother" ) );

			credits = false;
		}

		buttonPanel.repaint();
		mainPanel.repaint();
		validate();
	}

	/**
	 * Creates a panel containing all of the Runtime version information
	*/
	private void createVersionPanel()
	{
		version = new JPanel();
		ResourceBundle bundle = ResourceBundle.getBundle( "buildid" );

		version.setLayout( new GridLayout( 0, 2 ) );
                addItem( "TIM", "Tangible Instant Messenger" );
                addItem( "", "Eindhoven University of Technology" );
		addItem( "JBother Version", com.valhalla.jbother.JBother.JBOTHER_VERSION );
		addItem( "Build ID", bundle.getString( "build.number" ) );
		addItem( "Created By", "Adam Olsen" );
		addItem( "Smack Version", org.jivesoftware.smack.SmackConfiguration.getVersion() );
		addItem( "Host Operating System", System.getProperty( "os.name" ) + " " + System.getProperty( "os.version" ) );
		addItem( "Host System Architecture", System.getProperty( "os.arch" ) );
		addItem( "Java Version", System.getProperty( "java.version" ) );
		addItem( "Java Vendor", System.getProperty( "java.vendor" ) );
	}

	/**
	 * Adds a text label and a value
	 * @param name the label name
	 * @param the value
	*/
	private void addItem( String name, String value )
	{
		UIDefaults ui = UIManager.getDefaults();

		Font newFont = (Font)ui.get( "Label.font" );

		JLabel nameLabel = new JLabel( name + ":  ", SwingConstants.RIGHT );
		nameLabel.setFont( new Font( newFont.getName(), Font.BOLD, newFont.getSize() ) );

		JLabel valueLabel = new JLabel( value );
		version.add( nameLabel );
		version.add( valueLabel );
	}
}
