/*
 *  Copyright (C) 2003 Adam Olsen
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 1, or (at your option)
 *  any later version.
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
package com.valhalla.jbother;

import java.awt.*;
import java.awt.event.*;
import java.io.*;
import java.util.*;
import com.valhalla.jbother.groupchat.*;

import javax.swing.*;

import org.jivesoftware.smack.XMPPConnection;
import org.jivesoftware.smack.packet.Presence;

import com.valhalla.gui.*;
import com.valhalla.jbother.*;
import com.valhalla.jbother.menus.BuddyListTopMenu;
import com.valhalla.jbother.jabber.*;
import com.valhalla.jbother.plugins.events.*;
import com.valhalla.pluginmanager.*;
import com.valhalla.settings.Settings;
import com.valhalla.jbother.preferences.PreferencesDialog;

/**
 * BuddyList is the main controller for the buddy list, as as the buddy list is the main component
 * of the IM application it performs most of the work once it's been initialized.
 *
 * @author     Adam Olsen
 * @created    October 26, 2004
 * @version    1.0
 */
public class BuddyList extends JFrame
{
	private static BuddyList singleton = null;
	private ResourceBundle resources = ResourceBundle.getBundle( "JBotherBundle", Locale.getDefault() );
	private XMPPConnection connection;
	//the connection to the jabber server
	private BuddyListTree buddyListTree;
	//the inner tree - displays the buddies
	private Container container;

	//size and position
	private int preferredWidth = 150;
	private int preferredHeight = 300;
	private double x = 100;
	private double y = 100;

	private boolean signoff = false;

	private TabFrame tabFrame;
	private BuddyListTopMenu topBuddyMenu = new BuddyListTopMenu( this );
	private Hashtable buddyStatuses = null;
	private Presence.Mode currentMode = null;
	private String currentStatusString = resources.getString( "available" );
	private AwayHandler awayHandler = new AwayHandler();
	private javax.swing.Timer awayTimer = new javax.swing.Timer( 600000, awayHandler );

	private boolean idleAway = false;
	private Hashtable blockedUsers = new Hashtable();
	private static BuddyList buddyList = null;

	/**
	 * Constructor for the buddy list.  BuddyList is a singleton, so it's constructor
	 * is private
	 */
	private BuddyList()
	{

		super( "JBother" );
		setDefaultCloseOperation( DO_NOTHING_ON_CLOSE );

		ImageIcon icon = StatusIconCache.getStatusIcon( org.jivesoftware.smack.packet.Presence.Mode.AVAILABLE );
		if( icon != null )
		{
			setIconImage( icon.getImage() );
		}

		initComponents();
		//set up the visual components
		setJMenuBar( topBuddyMenu );
		pack();

		DefaultFocusManager myManager =
			new DefaultFocusManager()
			{
				public void processKeyEvent( Component focusedComponent, KeyEvent anEvent )
				{
					// Returning when you receive CTRL-TAB makes your components able to control that key
					if( anEvent.getKeyCode() == KeyEvent.VK_TAB &&
							( anEvent.getModifiers() & Toolkit.getDefaultToolkit().getMenuShortcutKeyMask() ) == Toolkit.getDefaultToolkit().getMenuShortcutKeyMask() )
					{

						if( anEvent.getID() == KeyEvent.KEY_RELEASED && tabFrame != null )
						{
							tabFrame.switchTab();
						}

						anEvent.consume();

						return;
					}

					super.processKeyEvent( focusedComponent, anEvent );
				}
			};

		FocusManager.setCurrentManager( myManager );
        try {
            jbInit();
        } catch (Exception ex) {
            ex.printStackTrace();
        }
    }

	/**
	 * Gets the BuddyList singleton
	 *
	 * @return        the BuddyList singleton
	 * @deprecated    Use getInstance() instead
	 */
	public static BuddyList getSingleton()
	{
		return getInstance();
	}

	/**
	 * Gets the BuddyList singleton
	 *
	 * @return    the BuddyList singleton
	 */
	public static BuddyList getInstance()
	{
		if( singleton == null )
		{
			singleton = new BuddyList();
		}
		return singleton;
	}

	/**
	 * Returns the top buddy menu
	 *
	 * @return    the buddy menu
	 */
	public BuddyListTopMenu getTopMenu()
	{
		return topBuddyMenu;
	}

	/**
	 * Starts the away timer
	 */
	public void startTimer()
	{
		// start the idle detection timer
		if( Settings.getInstance().getBoolean( "autoAway" ) )
		{
			com.valhalla.Logger.debug( "Starting away timer" );
			awayTimer.start();
		}

		buddyListTree.setConnection( this.connection );
		AWTEventListener listener = new MyAWTEventListener();
		Toolkit.getDefaultToolkit().addAWTEventListener( listener,
				AWTEvent.MOUSE_EVENT_MASK | AWTEvent.KEY_EVENT_MASK );
	}

	/**
	 * Loads the blocked users information
	 */
	protected void loadBlockedUsers()
	{
		File file = new File( JBother.profileDir + File.separatorChar + "blocked" );
		String line = "";

		try
		{
			FileReader fr = new FileReader( file );
			BufferedReader in = new BufferedReader( fr );

			while( ( line = in.readLine() ) != null )
			{
				blockedUsers.put( line, "blocked" );
				com.valhalla.Logger.debug( "Blocked> " + line );
			}

			fr.close();
		}
		catch( IOException e )
		{
			com.valhalla.Logger.debug( "Blocked users file was not found or could not be read." );
			return;
		}
	}

	/**
	 * Returns the away timer
	 *
	 * @return    the away timer
	 */
	public javax.swing.Timer getAwayTimer()
	{
		return awayTimer;
	}

	public void setIdleAway( boolean idleAway ) { this.idleAway = idleAway; }
	public boolean getIdleAway() { return idleAway; }
	public AwayHandler getAwayHandler() { return awayHandler; }

	/**
	 * Sets up the current connection
	 *
	 * @param  connection  the current connection
	 */
	public void init( XMPPConnection connection )
	{
		this.connection = connection;
		if( buddyStatuses == null ) buddyStatuses = new Hashtable();

		ConnectEvent event = new ConnectEvent( connection );
		PluginChain.fireEvent( event );

		buddyListTree.setConnection( connection );
		topBuddyMenu.getStatusMenu().setIcon( Presence.Mode.AVAILABLE );
	}

	/**
	 * Clears the buddy tree
	 */
	public void clearTree()
	{
		buddyListTree.clearBuddies();
	}

	/**
	 * initializes the buddy tree by loading the offline buddies
	 */
	public void initBuddies()
	{
		buddyListTree.loadOfflineBuddies();
	}

	/**
	 * Sets the current presence mode
	 *
	 * @param  mode  the mode to set it to
	 */
	public void setCurrentPresenceMode( Presence.Mode mode )
	{
		this.currentMode = mode;
                CurrentPresenceModeEvent event = new CurrentPresenceModeEvent( connection );
                PluginChain.fireEvent( event );

	}

	/**
	 * Sets the current status string
	 *
	 * @param  string  the string to use
	 */
	public void setCurrentStatusString( String string )
	{
		this.currentStatusString = string;
	}

	/**
	 * Gets the current presence mode
	 *
	 * @return    the current presence mode
	 */
	public Presence.Mode getCurrentPresenceMode()
	{
		return this.currentMode;
	}

	/**
	 * Returns the current status string
	 *
	 * @return    the current status string
	 */
	public String getCurrentStatusString()
	{
		return this.currentStatusString;
	}

	/**
	 * Returns the group chat frame
	 *
	 * @return    the group chat frame
	 */
	public TabFrame getTabFrame()
	{
		return tabFrame;
	}

	/**
	 * Starts the group chat frame
	 */
	public void startTabFrame()
	{
		if( tabFrame == null )
		{
			tabFrame = new TabFrame();
		}
	}

	/**
	 * checks to see if there are no more chat room windows in the groupchatframe
	 * if there are no more, the groupchatframe is destroyed
	 */
	public void stopTabFrame()
	{
		if( tabFrame == null )
		{
			return;
		}
		if( tabFrame.tabsLeft() <= 0 )
		{
			Point location = new Point( tabFrame.getLocationOnScreen() );
			Settings.getInstance().setProperty( "tabFrameX", new Double( location.getX() ).toString() );
			Settings.getInstance().setProperty( "tabFrameY", new Double( location.getY() ).toString() );

			DialogTracker.removeDialog( tabFrame );
			tabFrame = null;
			com.valhalla.Logger.debug( "Removing chat frame" );
		}
	}

	/**
	 * Removes a ChatPanel
	 *
	 * @param  panel  the chat panel to remove
	 */
	public void removeTabPanel( TabFramePanel panel )
	{
		if( tabFrame == null )
		{
			return;
		}

		tabFrame.removePanel( panel );
		stopTabFrame();
	}

	/**
	 * Adds a chat room window to the groupchat frame.
	 * If there is not groupchat frame one is created
	 *
	 * @param  panel   The feature to be added to the TabPanel attribute
	 */
	public void addTabPanel( TabFramePanel panel )
	{
		if( tabFrame == null )
		{
			tabFrame = new TabFrame();
		}
		tabFrame.addPanel( panel );

		if( !tabFrame.isVisible() )
		{
			tabFrame.setVisible( true );
			tabFrame.toFront();
		}

		panel.focusYourself();

		tabFrame.setStatus( getCurrentPresenceMode(), getCurrentStatusString() );
	}

	/**
	 * Gets the buddy status
	 *
	 * @param  userId  the user id of the BuddyStatus
	 * @return         The buddyStatus value
	 */
	public BuddyStatus getBuddyStatus( String userId )
	{
		if( buddyStatuses == null )
		{
			buddyStatuses = new Hashtable();
		}
		BuddyStatus buddy = new BuddyStatus( userId );

		if( buddyStatuses.get( userId.toLowerCase() ) != null )
		{
			buddy = (BuddyStatus)buddyStatuses.get( userId.toLowerCase() );
		}
		else
		{
			buddyStatuses.put( userId.toLowerCase(), buddy );
		}

		return buddy;
	}

	/**
	 * Returns all the buddy status that are available
	 *
	 * @return    the Hashtable containing all the buddy statuses
	 */
	public Hashtable getBuddyStatuses()
	{
		return this.buddyStatuses;
	}

	/**
	 * Returns the current connection
	 *
	 * @return    the current connection
	 */
	public XMPPConnection getConnection()
	{
		return this.connection;
	}

	/**
	 * Checks to see if a connection is active
	 * @return true if the connection is active (connected)
	*/
	public boolean checkConnection()
	{
		if( connection == null || !connection.isConnected() ) return false;
		else return true;
	}

	/**
	 * Displays a generic connection error
	*/
	public void connectionError()
	{
		Standard.warningMessage( null, resources.getString( "connectionError" ), resources.getString( "notConnected" ) );
	}

	/**
	 * Loads the saved settings from any previous settings
	 */
	protected void setPreferredLocation()
	{
		//load the settings from the settings file
		String xString = Settings.getInstance().getProperty( "buddyListX" );
		String yString = Settings.getInstance().getProperty( "buddyListY" );

		if( yString == null ) yString = "100";
		if( xString == null ) xString = "100";

		double x = Double.parseDouble( xString );
		double y = Double.parseDouble( yString );

		if( x < 0.0 ) x = 100.0;
		if( y < 0.0 ) y = 100.0;

		setLocation( (int)x, (int)y );
	}

	/**
	 * Saves the current settings - like the height and width of the buddy list
	 */
	public void saveSettings()
	{
		if( isVisible() )
		{
			//save the current buddy list size and position
			Dimension size = new Dimension( getSize() );
			Point location = new Point( getLocationOnScreen() );
			Settings.getInstance().setProperty( "buddyListX", new Double( location.getX() ).toString() );
			Settings.getInstance().setProperty( "buddyListY", new Double( location.getY() ).toString() );
			Settings.getInstance().setProperty( "buddyListWidth", new Integer( size.width ).toString() );
			Settings.getInstance().setProperty( "buddyListHeight", new Integer( size.height ).toString() );
		}

		//the "show" state settings
		Settings.getInstance().setBoolean( "showOfflineBuddies", buddyListTree.getShowOfflineBuddies() );
		Settings.getInstance().setBoolean( "showUnfiledBuddies", buddyListTree.getShowUnfiledBuddies() );
		Settings.getInstance().setBoolean( "showAgentBuddies", buddyListTree.getShowAgentBuddies() );
		Settings.getInstance().setBoolean( "showAgentMessages", buddyListTree.getShowAgentMessages() );
		Settings.getInstance().writeSettings();
	}

	/**
	 * Load saved settings from the last session and set the buddy list to the
	 * sizes that were saved.
	 */
	protected void setPreferredDimensions()
	{
		String width = Settings.getInstance().getProperty( "buddyListWidth" );
		String height = Settings.getInstance().getProperty( "buddyListHeight" );

		Dimension dim = new Dimension( 100, 100 );

		try {
			dim.setSize( Integer.parseInt( width ), Integer.parseInt( height ) );
		}
		catch( NumberFormatException ex ) { }

		setSize( dim );
	}

	/**
	 * Returns the buddy list tree
	 *
	 * @return    the buddy list tree
	 */
	public BuddyListTree getBuddyListTree()
	{
		return this.buddyListTree;
	}

	/**
	 * Init components just does some initial setup, sets the size and preferred location
	 * of the buddy list.  Sets up an new initial tree.
	 */
	private void initComponents()
	{
		container = getContentPane();
		container.setLayout( new BorderLayout() );

		buddyListTree = new BuddyListTree();

		container.add( buddyListTree, BorderLayout.CENTER );
		addWindowListener(
			new WindowAdapter()
			{
				public void windowClosing( WindowEvent e )
				{
					ExitingEvent event = new ExitingEvent( getInstance() );
					PluginChain.fireEvent( event );
					if( event.getExit() ) quitHandler();
				}

			} );
	}

	/**
	 * Sets whether or not we have signed off
	 *
	 * @param  value  true if we have signed off
	 */
	public void setSignoff( boolean value )
	{
		this.signoff = value;
	}

	/**
	 * Gets whether or not we have signed off
	 *
	 * @return    true if we have signed off
	 */
	public boolean getSignoff()
	{
		return signoff;
	}

	/**
	 * Gets the displayed name, or "me"
	 *
	 * @return    the myDisplayedName setting
	 */
	public String getMyName()
	{
		if( Settings.getInstance().getProperty( "myDisplayedName" ) != null )
		{
			return Settings.getInstance().getProperty( "myDisplayedName" );
		}
		return connection.getUser();
	}

	/**
	 * Listens for user input events.  If JBother is set to go "Away" on idle, this will restart
	 * the idle timer.
	 *
	 * @author     Adam Olsen
	 * @created    October 26, 2004
	 * @version    1.0
	 */
	public class MyAWTEventListener implements AWTEventListener
	{
		/**
		 * Called by the event listener
		 *
		 * @param  evt  the event
		 */
		public void eventDispatched( AWTEvent evt )
		{
			if( awayTimer.isRunning() )
			{
				awayTimer.restart();
			}
			else if( idleAway )
			{
				setStatus( Presence.Mode.AVAILABLE, getCurrentStatusString(), false );
				awayTimer.start();
				idleAway = false;
			}
		}
	}

	/**
	 * Closes the application
	 *
	 */
	public void quitHandler()
	{
		saveSettings();
		if( connection != null && connection.isConnected() ) connection.close();
		//and finally close the connection
		com.valhalla.Logger.closeLog();
		System.exit( 0 );
	}

	/**
	 * signs off, clears the buddy list
	*/
	public void signOff()
	{
		signoff = false;
		buddyListTree.clearBuddies();
		topBuddyMenu.getStatusMenu().setIcon( (Presence.Mode)null );
		connection.close();
		connection = null;
                setCurrentPresenceMode(null);

		if( buddyStatuses != null )
		{
			Iterator iterator = buddyStatuses.keySet().iterator();
			while( iterator.hasNext() )
			{
				String user = (String)iterator.next();
				BuddyStatus buddy = (BuddyStatus)buddyStatuses.get( user );
				buddy.resetBuddy();
				if( buddy.getConversation() != null )
				{
					((ChatPanel)buddy.getConversation()).disconnected();
				}
			}
		}

		if( tabFrame != null )
		{
			ArrayList remove = new ArrayList();

			JTabbedPane pane = tabFrame.getTabPane();
			for( int i = 0; i < pane.getTabCount(); i++ )
			{
				TabFramePanel panel = (TabFramePanel)pane.getComponentAt( i );
				if( panel instanceof ChatRoomPanel )
				{
					remove.add( panel );
				}
			}

			for( int i = 0; i < remove.size(); i++ )
			{
				TabFramePanel panel = (TabFramePanel)remove.get( i );
				removeTabPanel( panel );
				((ChatRoomPanel)panel).leave();
			}
		}
	}


	/**
	 * Close down the buddy list.
	 * Also closes down any other windows that might
	 * be open
	 */
	public void kill()
	{
		if( tabFrame != null )
		{
			tabFrame.leaveAll();
		}

		saveSettings();

		setVisible( false );

		currentMode = Presence.Mode.AVAILABLE;
		currentStatusString = resources.getString( "available" );
		connection = null;

		awayTimer.stop();

		Vector panels = MessageDelegator.getInstance().getPanels();
		int size = panels.size();
		for( int i = 0; i < size; i++ )
		{
			ConversationPanel panel = (ConversationPanel)panels.get( 0 );
			panel.closeHandler();
		}

		panels.removeAllElements();

		DialogTracker.kill();
		signoff = false;
	}

	/**
	 * Gets a list of blocked users
	 *
	 * @return    the list of blocked users
	 */
	public Hashtable getBlockedUsers()
	{
		return blockedUsers;
	}

	/**
	 * Sets the list of blocked users
	 *
	 * @param  users  the list of blocked users
	 */
	public void setBlockedUsers( Hashtable users )
	{
		this.blockedUsers = users;
	}

	/**
	 * Updates all the dialogs window icons
	 */
	public void updateIcons()
	{
		StatusIconCache.clearStatusIconCache();
		ImageIcon icon = StatusIconCache.getStatusIcon( org.jivesoftware.smack.packet.Presence.Mode.AVAILABLE );
		if( icon != null )
		{
			setIconImage( icon.getImage() );
			topBuddyMenu.getStatusMenu().setIcon( icon );

			DialogTracker tracker = DialogTracker.getInstance();
			for( int i = 0; i < tracker.size(); i++ )
			{
				if( tracker.get( i ) instanceof JFrame )
				{
					( (JFrame)tracker.get( i ) ).setIconImage( icon.getImage() );
				}
			}
		}
		topBuddyMenu.getStatusMenu().reloadStatusIcons();
	}

	/**
	 * Set the current status by sending a Jabber packet
	 *
	 * @param  mode            the mode to set it to
	 * @param  defaultMessage  the status message to set it to
	 * @param  getMessage      whether or not to get a new message
	 * @return                 true if the status packet was sent successfully
	 */
	public boolean setStatus( Presence.Mode mode, String defaultMessage, boolean getMessage )
	{
		String result;

		if( getMessage )
		{
			result = (String)JOptionPane.showInputDialog( null,
					resources.getString( "enterStatusMessage" ),
					resources.getString( "setStatus" ), JOptionPane.QUESTION_MESSAGE, null, null, defaultMessage );
		}
		else
		{
			result = defaultMessage;
		}

		if( result == null || result.equals( "" ) )
		{
			return false;
		}
		else
		{

			if( Settings.getInstance().getBoolean( "autoAway" ) )
			{
				if( mode == Presence.Mode.AVAILABLE )
				{
					awayTimer.start();
				}
				else
				{
					awayTimer.stop();
				}
			}

			int priority = 5;
			try
			{
				priority = Integer.parseInt( Settings.getInstance().getProperty( "priority" ) );
			}
			catch( NumberFormatException nfe )
			{
			}

			if( !checkConnection() )
			{
				Thread thread = new Thread( new ConnectorThread( mode, result, !false ) );
				thread.start();
				return true;
			}

			Presence presence = new Presence( Presence.Type.AVAILABLE, result, priority, mode );

			connection.sendPacket( presence );

			setCurrentPresenceMode( mode );
			if( !idleAway )
			{
				setCurrentStatusString( result );
			}

			// if the group chat window is open, set the status there too
			if( getTabFrame() != null )
			{
				getTabFrame().setStatus( mode, result );
			}

			ParsedBuddyInfo info = new ParsedBuddyInfo( connection.getUser() );
			BuddyStatus buddy = BuddyList.getInstance().getBuddyStatus( info.getUserId() );
			buddy.addResource( info.getResource(), priority, mode, result );
			topBuddyMenu.getStatusMenu().loadSelfStatuses();

			updateIcons();
		}

		return true;
	}

    private void jbInit() throws Exception {
    }
}

/**
 * Sets the user to away after a specific amount of idle time
 *
 * @author     Adam Olsen
 * @created    October 26, 2004
 * @version    1.0
 */
class AwayHandler implements ActionListener
{
	private ResourceBundle resources = ResourceBundle.getBundle( "JBotherBundle", Locale.getDefault() );

	/**
	 * Called by the enclosing event listener
	 *
	 * @param  e  the event
	 */
	public void actionPerformed( ActionEvent e )
	{
		BuddyList.getInstance().setIdleAway( true );
		BuddyList.getInstance().setStatus( Presence.Mode.AWAY, resources.getString( "autoAway" ), false );
		BuddyList.getInstance().getAwayTimer().stop();
	}
}
