/*
	Copyright (C) 2003 Adam Olsen

	This program is free software; you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation; either version 1, or (at your option)
	any later version.

	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with this program; if not, write to the Free Software
	Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/

package com.valhalla.jbother;

import java.awt.*;
import java.net.URL;
import java.util.*;
import java.util.regex.Pattern;
import java.text.*;

import javax.swing.*;
import javax.swing.tree.*;

import org.jivesoftware.smack.*;
import org.jivesoftware.smack.packet.*;


import com.valhalla.gui.*;
import com.valhalla.jbother.*;
import com.valhalla.jbother.jabber.BuddyStatus;
import com.valhalla.settings.Settings;

import org.dotuseful.ui.tree.*;

/**
 * The renderer for the buddy list
 * @author Adam Olsen
 * @version 1.0
*/
class BuddyListRenderer extends DefaultTreeCellRenderer
{
	private ResourceBundle resources = ResourceBundle.getBundle( "JBotherBundle", Locale.getDefault() );

	/**
	 * Gets an individual cell
	 * @see <code>javax.swing.DefaultTreeCellRenderer</code>
	*/
	public synchronized Component getTreeCellRendererComponent(
		JTree tree, Object value, boolean sel, boolean expanded, boolean leaf, int row, boolean hasFocus
	)
	{
		AutomatedTreeNode node = (AutomatedTreeNode)value;
		String stringVal = node.getUserObject().toString();

		UIDefaults ui = UIManager.getDefaults();

		if( node.getUserObject() instanceof BuddyStatus )
		{
			// sets up the buddy information
			BuddyStatus buddy = (BuddyStatus)node.getUserObject();

			value = buddy.getUser();
			if( buddy.getName() != null ) value = buddy.getName();
			stringVal = value.toString();

			String user = buddy.getUser();
			String server = buddy.getUser();
			if( Pattern.matches( ".*@.*", user ) )
			{
				String parts[] = new String[2];
				parts = buddy.getUser().split( "@" );
				user = parts[0];
				server = parts[0];
				if( parts[1] != null ) server = parts[1];
			}

			// gets all the resources
			int resourceCount = 0;
			StringBuffer resources = new StringBuffer();

			Iterator i = buddy.keySet().iterator();

			while( i.hasNext() )
			{
				String key = (String)i.next();

				if( !key.equals( "N/A" ) )
				{
					boolean add = false;

					if( key.equals( buddy.getHighestResource() ) ) add = true;
					else resources .append("  ");

					resources.append(key).append(" (").append(buddy.get( key )).append(")");
					if( add ) resources.append(" <b>*</b>");
					if( i.hasNext() ) resources.append("<br>");
					resourceCount++;
				}
			}

			// draw the tooltip
			StringBuffer tooltip = new StringBuffer();
			tooltip.append("<html><table border='0'><tr><td colspan='2'><b><font size='+1'>")
				.append(user.replaceAll( "\\%", "@" ))
				.append("</font></b><table border='0' cellpadding='2' cellspacing='2'><tr><td nowrap><b>")
				.append(this.resources.getString( "server" )).append(":</b></td><td nowrap>")
				.append(server).append("</td></tr>");

			if( resourceCount > 0 ) tooltip.append("<tr><td valign=\"top\" nowrap><b>")
				.append(this.resources.getString( "pluralResources" ))
				.append(":</b></td><td nowrap>").append(resources).append("</td></tr>");

			String statusMessage = buddy.getStatusMessage( buddy.getHighestResource() );
			if( statusMessage != null && !statusMessage.equals( "" ) )
			{
				tooltip.append("<tr><td nowrap><b>").append(this.resources.getString( "status" ))
					.append(":</b></td><td nowrap>").append(statusMessage).append("</td></tr>");
			}

			String using = buddy.getVersionInfo();
			if( using != null )
			{
				tooltip.append("<tr><td nowrap><b>").append(this.resources.getString( "using" ))
					.append(":</b></td><td nowrap>").append(using).append("</td></tr>");
			}

			tooltip.append("</table></td></tr></table></html>");
			setToolTipText( tooltip.toString() );
			setFont( (Font)ui.get( "Label.font" ) );

			super.getTreeCellRendererComponent( tree, stringVal, sel, expanded, leaf, row, hasFocus );

			// setting up status icon
			Presence.Mode mode = null;
			if( buddy.size() == 0 )
			{
				setForeground( Color.GRAY );
			}
			else mode = buddy.getPresence( buddy.getHighestResource() );

			ImageIcon statusIcon = StatusIconCache.getStatusIcon( mode );
			if( statusIcon != null ) setIcon( statusIcon );
			// /setting up status icon

		}

		/**
		 * Draws the groups
		*/
		else {
			Font newFont = (Font)ui.get( "Label.font" );
			setFont( new Font( newFont.getName(), Font.BOLD, newFont.getSize() ) );

			super.getTreeCellRendererComponent( tree, stringVal, sel, expanded, leaf, row, hasFocus );

			setToolTipText( null );
			setIcon( null );
		}

		return this;
	}
}
