/*
	Copyright (C) 2003 Adam Olsen

	This program is free software; you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation; either version 1, or (at your option)
	any later version.

	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with this program; if not, write to the Free Software
	Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/

package com.valhalla.jbother;

import java.io.File;
import java.util.*;
import org.jivesoftware.smack.*;
import com.valhalla.settings.*;

import com.valhalla.gui.*;
import javax.swing.*;
import java.awt.*;
import java.awt.event.*;
import java.lang.reflect.*;
import javax.swing.border.*;
import com.valhalla.jbother.*;

/**
 * Displays a Dialog allowing the user to change his passowrd on the Jabber server
 *
 * @author Adam Olsen
 * @version 1.0
*/
public class ChangePasswordDialog extends JDialog
{
	private ResourceBundle resources = ResourceBundle.getBundle( "JBotherBundle", Locale.getDefault() );
	private JPanel mainPanel;
	private JButton okButton = new JButton( resources.getString( "okButton" ) );
	private JButton cancelButton = new JButton( resources.getString( "cancelButton" ) );
	private JPasswordField passwordField = new JPasswordField( 16 );
	private JPasswordField verifyPasswordField = new JPasswordField( 16 );
	private ChangePasswordDialog thisPointer = this;
	private WaitDialog wait = new WaitDialog( resources.getString( "pleaseWait" ) );

	/**
	 * Sets up the dialog
	*/
	public ChangePasswordDialog()
	{
		setTitle( resources.getString( "changePassword" ) );
		initComponents();
		DialogTracker.addDialog( this, true, true );
		pack();
		setLocationRelativeTo( null );
	}

	/**
	 * Sets up the various visual components
	*/
	private void initComponents()
	{
		mainPanel = (JPanel)getContentPane();

		mainPanel.setBorder( BorderFactory.createTitledBorder( resources.getString( "changePassword" ) ) );
		mainPanel.setLayout( new BorderLayout() );

		JPanel buttonPanel = new JPanel();
		buttonPanel.setLayout( new BoxLayout( buttonPanel, BoxLayout.X_AXIS ) );
		buttonPanel.add( Box.createHorizontalGlue() );
		buttonPanel.add( cancelButton );
		buttonPanel.add( okButton );
		buttonPanel.add( Box.createHorizontalGlue() );
		buttonPanel.setBorder( BorderFactory.createEmptyBorder( 5, 5, 5, 5 ) );

		mainPanel.add( buttonPanel, BorderLayout.SOUTH );

		JPanel innerPanel = new JPanel();
		innerPanel.setLayout( new BoxLayout( innerPanel, BoxLayout.Y_AXIS ) );

		JPanel passwordPanel = new JPanel();
		passwordPanel.setLayout( new BoxLayout( passwordPanel, BoxLayout.X_AXIS ) );
		passwordPanel.add( new JLabel( resources.getString( "newPassword" ) + ":   ", SwingConstants.RIGHT ) );
		passwordPanel.add( passwordField );
		passwordPanel.setBorder( BorderFactory.createEmptyBorder( 0, 5, 2, 5 ) );
		innerPanel.add( passwordPanel );

		JPanel verifyPanel = new JPanel();
		verifyPanel.setLayout( new BoxLayout( verifyPanel, BoxLayout.X_AXIS ) );
		verifyPanel.add( new JLabel( resources.getString( "verifyPassword" ) + ": ", SwingConstants.RIGHT ) );
		verifyPanel.add( verifyPasswordField );
		verifyPanel.setBorder( BorderFactory.createEmptyBorder( 0, 5, 5, 5 ) );
		innerPanel.add( verifyPanel );

		// we have to set the JPasswordField fonts manually for some reason
		passwordField.setFont( okButton.getFont() );
		verifyPasswordField.setFont( okButton.getFont() );

		passwordField.grabFocus();

		mainPanel.add( innerPanel, BorderLayout.CENTER );
		addListeners();
	}

	/**
	 * Adds listeners to the dialogs buttons
	*/
	private void addListeners()
	{
		cancelButton.addActionListener( new ActionListener()
		{
			public void actionPerformed( ActionEvent e ) { DialogTracker.removeDialog( thisPointer ); }
		} );

		okButton.addActionListener( new ActionListener()
		{
			public void actionPerformed( ActionEvent e ) { okHandler(); }
		} );

	}

	/**
	 * Checks the information and runs the PasswordChangeThread
	*/
	private void okHandler()
	{
		String pass = new String( passwordField.getPassword() );
		String verify = new String( verifyPasswordField.getPassword() );

		if( pass.equals( "" ) )
		{
			Standard.warningMessage( this, resources.getString( "changePassword" ), resources.getString( "passwordRequired" ) );
			return;
		}

		if( !verify.equals( pass ) )
		{
			Standard.warningMessage( this, resources.getString( "changePassword" ), resources.getString( "verificationMatch" ) );
			return;

		}

		wait.show();
		setVisible( false );
		Thread thread = new Thread( new PasswordChangeThread( pass ) );
		thread.start();
	}

	/**
	 * Sends the new password to the server and gets the response
	 * @author Adam Olsen
	 * @version 1.0
	*/
	class PasswordChangeThread implements Runnable
	{
		private String newPass;
		public PasswordChangeThread( String p ) { newPass = p; }
		public void run()
		{
			String errorMessage = null;

			if( BuddyList.getInstance().checkConnection() )
			{
				AccountManager manager = BuddyList.getInstance().getConnection().getAccountManager();
				try {
					manager.changePassword( newPass );
				}
				catch( XMPPException e )
				{
					errorMessage = e.getMessage();
				}
			}
			else errorMessage = resources.getString( "notConnected" );

			wait.dispose();

			if( errorMessage == null )
			{
				if( Settings.getInstance().getProperty( "password" ) != null )
					Settings.getInstance().setProperty( "password", newPass );
				Standard.noticeMessage( thisPointer, resources.getString( "changePassword" ), resources.getString( "passwordChanged" ) );
			}
			else {
				Standard.warningMessage( thisPointer, resources.getString( "changePassword" ), errorMessage );
			}

			DialogTracker.removeDialog( thisPointer );
		}
	}
}
