/*
 Copyright (C) 2003 Adam Olsen

 This program is free software; you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
 the Free Software Foundation; either version 1, or (at your option)
 any later version.

 This program is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU General Public License for more details.

 You should have received a copy of the GNU General Public License
 along with this program; if not, write to the Free Software
 Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

package com.valhalla.jbother;

import java.awt.*;
import java.awt.event.*;
import java.net.URL;
import java.util.*;
import java.text.*;

import javax.swing.*;
import javax.swing.event.*;
import java.beans.*;

import org.jivesoftware.smack.packet.Message;
import org.jivesoftware.smack.packet.Presence;

import com.valhalla.gui.*;
import com.valhalla.jbother.*;
import com.valhalla.jbother.menus.ConversationPopupMenu;
import com.valhalla.jbother.jabber.BuddyStatus;
import com.valhalla.settings.Settings;
import com.valhalla.jbother.plugins.events.ConversationEvent;

/**
 * Handles conversations between two users.
 * It is usually associated with a BuddyStatus.
 *
 * @author Adam Olsen
 * @version 1.0
 * @see com.valhalla.jbother.jabber.BuddyStatus
 **/
public class ChatPanel extends ConversationPanel {

    private ResourceBundle resources = ResourceBundle.getBundle("JBotherBundle",
            Locale.getDefault());
    //private StringBuffer conversationText = new StringBuffer();
    private boolean offlineMessage = false;
    private ConversationPopupMenu popMenu = new ConversationPopupMenu(this,
            conversationArea);

    //private HTMLDocument document = (HTMLDocument)conversationArea.getDocument();
    private JSplitPane container;
    private JScrollPane conversationScroll;
    private JPanel buttonPanel = new JPanel();
    private JPanel scrollPanel = new JPanel(new GridLayout(1, 0));
    private JTextArea textEntryArea = new JTextArea();
    //private JCheckBox useHTML = new JCheckBox( "Use HTML" );
    // for logging
    private JComboBox resourceBox = new JComboBox();
    private JButton clearButton = new JButton(Standard.getIcon(
            "images/buttons/New24.gif"));
    private JButton logButton = new JButton(Standard.getIcon(
            "images/buttons/Log24.gif"));
    private JButton emoteButton = new JButton(Standard.getIcon(
            "images/buttons/smiley.gif"));
    private ChatPanel thisPointer = this;
    private boolean divSetUp = false;

    /**
     * Sets up the ChatPanel - creates all visual components and adds event listeners
     * @param buddy the buddy to associate with
     * @param userId the buddy's userId
     * @param buddyName the buddy's alias
     */
    public ChatPanel(BuddyStatus buddy) {
        super(buddy);

        setLayout(new BoxLayout(this, BoxLayout.Y_AXIS));
        setBorder(BorderFactory.createEmptyBorder(5, 5, 5, 5));

        //create two fields, one for where you type your message to be sent, and the
        //other where you see the conversation that has already happened.
        textEntryArea.setLineWrap(true);
        textEntryArea.setWrapStyleWord(true);

        conversationScroll = new JScrollPane(conversationArea);
        conversationScroll.setVerticalScrollBarPolicy(JScrollPane.
                VERTICAL_SCROLLBAR_ALWAYS);
        conversationArea.setScroll(conversationScroll);

        scrollPanel.add(conversationScroll);

        container = new JSplitPane(JSplitPane.VERTICAL_SPLIT, scrollPanel,
                                   new JScrollPane(textEntryArea));
        container.setResizeWeight(1);

        JPanel containerPanel = new JPanel();
        containerPanel.setLayout(new BoxLayout(containerPanel, BoxLayout.X_AXIS));
        containerPanel.add(container);

        JPanel bottomPanel = new JPanel();
        bottomPanel.setLayout(new BoxLayout(bottomPanel, BoxLayout.Y_AXIS));

        if (buddy.getUser().indexOf("/") >= 0) {
            resourceBox.setEnabled(false);
        }
        resourceBox.setRenderer(new PresenceComboBoxRenderer());
        buttonPanel.setLayout(new BoxLayout(buttonPanel, BoxLayout.X_AXIS));

        JPanel resourcePanel = new JPanel();
        resourcePanel.setLayout(new BoxLayout(resourcePanel, BoxLayout.Y_AXIS));
        resourcePanel.add(Box.createVerticalGlue());
        resourcePanel.add(resourceBox);

        buttonPanel.add(resourcePanel);
        buttonPanel.add(Box.createRigidArea(new Dimension(5, 0)));
        emoteButton.setPreferredSize(new Dimension(26, 26));
        buttonPanel.add(emoteButton);
        clearButton.setPreferredSize(new Dimension(26, 26));
        buttonPanel.add(clearButton);
        logButton.setPreferredSize(new Dimension(26, 26));
        buttonPanel.add(logButton);

        bottomPanel.add(buttonPanel);

        add(containerPanel);
        add(Box.createRigidArea(new Dimension(0, 5)));
        add(bottomPanel);

        textEntryArea.grabFocus();

        addListeners();
        updateResources();

        ConversationEvent event = new ConversationEvent(this);
        event.setType(ConversationEvent.Type.INIT);
        com.valhalla.pluginmanager.PluginChain.fireEvent(event);

    }

    public JPanel getButtonPanel() {
        return buttonPanel;
    }

    /**
     * @return the input area of this panel
     */
    public Component getInputComponent() {
        return textEntryArea;
    }

    /**
     * Sets up the Divider
     */
    public void setUpDivider() {
        if (divSetUp) {
            return;
        }
        divSetUp = true;
        String stringHeight = Settings.getInstance().getProperty(
                "conversationWindowHeight");

        // set up the divider location from settings
        String divLocString = Settings.getInstance().getProperty(
                "conversationWindowDividerLocation");
        int divLoc = 0;

        if (divLocString != null) {
            divLoc = Integer.parseInt(divLocString);
        } else {
            divLoc = Integer.parseInt(stringHeight) - 117;
        }

        container.setDividerLocation(divLoc);
        container.validate();
        container.addPropertyChangeListener("lastDividerLocation",
                                            new DividerListener());
    }

    /**
     * Focuses when a Tab in the TabFrame is clicked
     */
    public void focusYourself() {
        textEntryArea.grabFocus();
    }

    /**
     * @return the ChatPanel's JSPlitPane
     */
    public JSplitPane getSplitPane() {
        return container;
    }

    /**
     * Listens for the user to move the divider, and saves it's location
     * @author Adam Olsen
     * @version 1.0
     */
    private class DividerListener implements PropertyChangeListener {
        public void propertyChange(PropertyChangeEvent e) {
            Settings.getInstance().setProperty(
                    "conversationWindowDividerLocation",
                    e.getOldValue().toString());
        }
    }


    /**
     * Updates the ConversationArea's font
     * @param font the font to update to
     */
    public void updateStyle(Font font) {
        conversationArea.loadStyleSheet(font);
    }

    /**
     * Gets the ComboBox with all the buddy's resources
     * @return the ComboBox
     */
    public JComboBox getResourceBox() {
        return resourceBox;
    }

    /**
     * Updates the JComboBox with the buddy's current resources
     **/
    public void updateResources() {
        SwingUtilities.invokeLater(new Runnable() {
            public void run() {
                String selected = (String) resourceBox.getSelectedItem();
                if (selected == null) {
                    selected = "";
                }

                resourceBox.removeAllItems();
                resourceBox.addItem(resources.getString("defaultResource"));
                resourceBox.addItem(resources.getString("allResources"));

                if (buddy.getUser().indexOf("/") >= 0) {
                    return;
                }

                Iterator i = buddy.keySet().iterator();

                int count = 2, sel = 0;
                while (i.hasNext()) {
                    String key = (String) i.next();
                    if (!key.equals("N/A")) {
                        resourceBox.addItem(key);
                        if (key.equals(selected)) {
                            sel = count;
                        }
                        count++;
                    }
                }

                if (count == 2) {
                    sel = 0;
                    resourceBox.setEnabled(false);
                } else if (count == 3) {
                    sel = 2;
                    resourceBox.setEnabled(false);
                } else {
                    resourceBox.setEnabled(true);
                }

                if (selected.equals(resources.getString("allResources"))) {
                    sel = 1;
                }
                if (sel > 0 && sel <= resourceBox.getItemCount()) {
                    resourceBox.setSelectedIndex(sel);
                } else {
                    resourceBox.setSelectedIndex(0);
                }
                resourceBox.repaint();
            }
        });
    }

    /**
     * Adds the various event listeners for the components that are a part of this
     * frame
     **/
    private void addListeners() {
        clearButton.setToolTipText(resources.getString("clear"));
        logButton.setToolTipText(resources.getString("viewLog"));
        emoteButton.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent e) {
                JFrame f = frame;
                if (f == null) {
                    f = BuddyList.getInstance().getTabFrame();
                }
                Emoticons.getInstance().displayEmoticonChooser(f, emoteButton,
                        textEntryArea);
            }
        });

        clearButton.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent e) {
                conversationArea.setText("");
            }
        });

        logButton.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent e) {
                openLogWindow();
            }
        });

        //if the press enter, send the message
        Action sendMessageAction = new AbstractAction() {
            public void actionPerformed(ActionEvent e) {
                sendHandler();
            }
        };

        Action shiftEnterAction = new AbstractAction() {
            public void actionPerformed(ActionEvent e) {
                int pos = textEntryArea.getCaretPosition();
                if (pos < 0) {
                    pos = 0;
                }

                textEntryArea.setText(textEntryArea.getText() + "\n");

                try {
                    textEntryArea.setCaretPosition(pos + 1);
                } catch (IllegalArgumentException ex) {}
            }
        };

        Action checkCloseAction = new AbstractAction() {
            public void actionPerformed(ActionEvent e) {
                checkCloseHandler();
            }
        };

        Action closeAction = new AbstractAction() {
            public void actionPerformed(ActionEvent e) {
                closeHandler();
            }
        };

        //for making it so they can select text in the window and not have it lose focus on them
        conversationArea.addMouseMotionListener(new MouseMotionAdapter() {
            public void mouseDragged(MouseEvent e) {
                conversationArea.grabFocus();
            }
        });

        conversationArea.addFocusListener(new FocusListener() {
            public void focusGained(FocusEvent e) {
                //only request focus to the input box if there is no text selected
                if (conversationArea.getSelectedText() == null) {
                    textEntryArea.requestFocus();
                } else {
                    conversationArea.grabFocus();
                }
            }

            public void focusLost(FocusEvent e) {}
        });

        conversationArea.addMouseListener(new RightClickListener(popMenu));

        textEntryArea.getInputMap().put(KeyStroke.getKeyStroke(KeyEvent.
                VK_ENTER, 0), sendMessageAction);
        textEntryArea.getInputMap().put(KeyStroke.getKeyStroke(KeyEvent.
                VK_ENTER, java.awt.event.InputEvent.SHIFT_MASK),
                                        shiftEnterAction);
        textEntryArea.getInputMap().put(KeyStroke.getKeyStroke(KeyEvent.
                VK_ENTER, Toolkit.getDefaultToolkit().getMenuShortcutKeyMask()),
                                        shiftEnterAction);
        textEntryArea.getInputMap().put(KeyStroke.getKeyStroke(KeyEvent.VK_W,
                Toolkit.getDefaultToolkit().getMenuShortcutKeyMask()),
                                        checkCloseAction);
        textEntryArea.getInputMap().put(KeyStroke.getKeyStroke(KeyEvent.VK_K,
                Toolkit.getDefaultToolkit().getMenuShortcutKeyMask()),
                                        closeAction);
    }

    /**
     * Displays a little blurb that the last messaged recieved was from an offline buddy
     */
    public void setOfflineMessage() {
        if (!offlineMessage) {
            conversationArea.setText("<b>" +
                                     resources.getString("offlineMessage") +
                                     ":</b><hr width='100%'>");
            offlineMessage = true;
        }
    }

    /**
     * Displays a message in the window when the buddy signs off
     */
    public void signedOff() {
        conversationArea.append("<font color='green'>" + getDate() + "<b> " +
                                buddy.getName() + " " +
                                resources.getString("signedOff") +
                                "</b></font><br>");
        if (logOut != null) {
            logOut.println(conversationArea.getLastHTML());
        }
    }

    /**
     * Displays a message in the window when a buddy signs on
     */
    public void signedOn() {
        conversationArea.append("<font color='green'>" + getDate() + "<b> " +
                                buddy.getName() + " " +
                                resources.getString("signedOn") +
                                "</b></font><br>");
        if (logOut != null) {
            logOut.println(conversationArea.getLastHTML());
        }
    }

    /**
     * Displays a "disconnected" message"
     */
    public void disconnected() {
        conversationArea.append("<b>" + getDate() + " **** " +
                                resources.getString("disconnected"));
    }

    public void displayInfo(String info){
        conversationArea.append(info);
    }

    /**
     * Recieves a message
     * @param sbj the message subject
     * @param body the message body
     * @param resource the resource the message came from if there is one
     */
    public void recieveMessage(String sbj, String body, final String resource) {
        SwingUtilities.invokeLater(new Runnable() {
            public void run() {
                if (resource != null && buddy.getUser().indexOf("/") < 0) {
                    resourceBox.setSelectedItem(resource);
                }
                resourceBox.validate();
            }
        });

        ConversationEvent event = new ConversationEvent(this);
        event.setType(ConversationEvent.Type.RECEIVEING_MESSAGE);
        event.setContent(body);
        com.valhalla.pluginmanager.PluginChain.fireEvent(event);

        body = event.getContent();
        if (!event.isNoMoreFormatting()) {
            body = ConversationText.replaceText(body, true);
        }
        if (body.startsWith(" /me ")) {
            body = body.replaceAll("^ \\/me ", "");
            body = (getDate() + " <b>* " +
                    buddy.getName() + "</b> " + body +
                    "<br>");
        } else {
            body = ("<font color='#16569e'>" + getDate() +
                    " <b>" + buddy.getName() +
                    "</b></font>: " +
                    body + "<br>");
        }

        if (!event.isToHide()) {
            conversationArea.append(body);
        }

        if (logOut != null) {
            logOut.println(conversationArea.getLastHTML());
        }

        super.recieveMessage();
        setUpDivider();
    }

    /**
     * Sends the message in the TextEntryArea
     **/
    private void sendHandler() {
        if (!textEntryArea.getText().equals("")) {
            if (!BuddyList.getInstance().checkConnection()) {
                BuddyList.getInstance().connectionError();
                return;
            }

            ConversationEvent event = new ConversationEvent(this);
            event.setType(ConversationEvent.Type.SENDING_MESSAGE);
            event.setContent(textEntryArea.getText());

            com.valhalla.pluginmanager.PluginChain.fireEvent(event);

            sendBuddyMessage(event.getContent());

            String text = event.getContent();
            if (!event.isNoMoreFormatting()) {
                text = ConversationText.replaceText(text, true);
            }

            if (text.startsWith(" /me ")) {
                text = text.replaceAll("^ \\/me ", "");
                text = (getDate() + " <b>* " +
                        BuddyList.getInstance().getMyName() +
                        "</b> " + text + "</font><br>");
            } else {
                text = ("<font color='maroon'>" + getDate() +
                        " <b>" +
                        BuddyList.getInstance().getMyName() +
                        "</b></font>: " + text + "<br>");
            }

            if (!event.isToHide()) {
                conversationArea.append(text);
            }

            textEntryArea.setText("");
            //com.valhalla.jbother.sound.SoundPlayer.play( "sentSound" );
            if (logOut != null) {
                logOut.println(conversationArea.getLastHTML());
            }
        }
    }

    public void directSend(String text, boolean show) {
        if (!BuddyList.getInstance().checkConnection()) {
            BuddyList.getInstance().connectionError();
            return;
        }

        sendBuddyMessage(text);

        if (show) {
            text = ConversationText.replaceText(text, true);

            if (text.startsWith(" /me ")) {
                text = text.replaceAll("^ \\/me ", "");
                text = (getDate() + " <b>* " +
                        BuddyList.getInstance().getMyName() +
                        "</b> " + text + "</font><br>");
            } else {
                text = ("<font color='maroon'>" + getDate() +
                        " <b>" +
                        BuddyList.getInstance().getMyName() +
                        "</b></font>: " + text + "<br>");
            }

            conversationArea.append(text);

        }

        if (logOut != null) {
            logOut.println(conversationArea.getLastHTML());
        }
    }

    /**
     * Sends the message to the resource in the JComboBox
     * @param text the message to send
     */
    public void sendBuddyMessage(String text) {
        String to = buddy.getUser();
        int sel = resourceBox.getSelectedIndex();

        // if they've selected a resource, send to it
        if (sel != 0 && sel != 1 && sel != -1) {
            to += "/" + (String) resourceBox.getSelectedItem();
        }

        ArrayList send = new ArrayList();

        if (sel != 1 || resourceBox.getItemCount() <= 2) {
            send.add(to);
        }

        // if they've selected to send to all resources, send to all
        else {
            Set keys = buddy.keySet();
            Iterator i = keys.iterator();
            while (i.hasNext()) {
                String key = (String) i.next();
                if (!key.equals("N/A")) {
                    send.add(buddy.getUser() + "/" + key);
                }
            }
        }

        for (int i = 0; i < send.size(); i++) {
            Message message = new Message();

            message.setBody(text);
            message.setType(Message.Type.CHAT);

            message.setTo((String) send.get(i));

            if (BuddyList.getInstance().checkConnection()) {
                BuddyList.getInstance().getConnection().sendPacket(message);
            } else {
                BuddyList.getInstance().connectionError();
            }
        }
    }

    /**
     * Creates the containing frame
     */
    public void createFrame() {
        frame = new JFrame();
        frame.setContentPane(this);
        frame.pack();

        ImageIcon icon = StatusIconCache.getStatusIcon(org.jivesoftware.smack.
                packet.Presence.Mode.AVAILABLE);
        if (icon != null) {
            frame.setIconImage(icon.getImage());
        }

        frame.addWindowListener(
                new WindowAdapter() {
            public void windowClosing(WindowEvent e) {
                if (Settings.getInstance().getProperty("preserveMessages") == null) {
                    closeHandler();
                } else {
                    startTimer();
                    frame.setVisible(false);
                }
            }
        });

        frame.setTitle(buddy.getName() + " (" + buddy.getUser() + ")");
        frame.pack();

        String stringWidth = Settings.getInstance().getProperty(
                "conversationWindowWidth");
        String stringHeight = Settings.getInstance().getProperty(
                "conversationWindowHeight");

        if (stringWidth == null) {
            stringWidth = "400";
        }
        if (stringHeight == null) {
            stringHeight = "340";
        }

        frame.setSize(new Dimension(Integer.parseInt(stringWidth),
                                    Integer.parseInt(stringHeight)));

        // add a resize window listener
        frame.addComponentListener(new ComponentAdapter() {
            public void componentResized(ComponentEvent e) {
                Dimension dim = frame.getSize();
                Settings.getInstance().setProperty("conversationWindowWidth",
                        new Integer((int) dim.getWidth()).toString());
                Settings.getInstance().setProperty("conversationWindowHeight",
                        new Integer((int) dim.getHeight()).toString());
            }
        });
        Standard.cascadePlacement(frame);

        setUpDivider();
        frame.setVisible(true);
    }

    /**
     * This renders the resource combo box - and displays icons for the online status
     * of each resource
     * @author Adam Olsen
     * @version 1.0
     */
    class PresenceComboBoxRenderer extends JLabel implements ListCellRenderer {
        public Component getListCellRendererComponent(JList list, Object value,
                int index,
                boolean isSelected, boolean cellHasFocus) {

            Presence.Mode mode = null;

            if (buddy.size() != 0) {
                mode = buddy.getPresence(buddy.getHighestResource());
            }

            if (value.toString().equals(resources.getString("allResources"))) {
                mode = Presence.Mode.AVAILABLE;
            }

            if (!value.toString().equals(resources.getString("defaultResource")) &&
                !value.toString().equals(resources.getString("allResources"))) {
                mode = buddy.getPresence(value.toString());
            }

            ImageIcon icon = StatusIconCache.getStatusIcon(mode);
            if (icon != null) {
                setIcon(icon);
            }
            setText(value.toString());

            return this;
        }
    }
}
