/*
    Copyright (C) 2003 Adam Olsen

	This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 1, or (at your option)
    any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/

package com.valhalla.jbother;

/**
 * Replaces conversation text like URLS with links
 * @author Adam Olsen
 * @version 1.0
*/
public class ConversationText
{
	/**
	 * Replaces the text so it can better be displayed in the ConversationArea
	 * @param text the text to replace
	 * @param asHTML not used yet
	*/
	public static String replaceText( String text, boolean asHTML )
	{
		text = " " + text + " ";

		if( !asHTML )
		{
			text = text.replaceAll( "\\&", "&amp;" );
			text = text.replaceAll( "\\<", "&lt;" );
			text = text.replaceAll( "\\>", "&gt;" );
		}
		//else {*/
			// JEditorPane doesn't support XHTML so well, so get rid of end tags that look like />
			//text = text.replaceAll( "\\/\\>", ">" );
		/*}*/

		// replace _word_ with <u>_word_</u>
		text = text.replaceAll( "(^| )(_[A-Za-z0-9]+_)(\\.|$| )", "$1<u>$2</u>$3" );

		// replace /word/ with <i>/word/</i>
		text = text.replaceAll( "(^| )(/[A-Za-z0-9]+/)(\\.|$| )", "$1<i>$2</i>$3" );

		// replace *word* with <b>*word*</b>
		text = text.replaceAll( "(^| )(\\*[A-Za-z0-9]+\\*)(\\.|$| )", "$1<b>$2</b>$3" );

		// replace emoticon symbols
		text = Emoticons.getInstance().replaceIcons( text );

		// replace URLs and email addresses
		text = text.replaceAll( "(\\s|^)((?!(ftp|http|https)://)[^\\s\"']+?@[^\\s\"']+?)(\\s|$)",
			"$1<a href=\"mailto:$2\">$2</a>$4" );
		text = text.replaceAll( "(^|\\s)((ftp|http|https)://[^\\s\"']+?)(\\s|$)", "$1<a href=\"$2\">$2</a>$4" );

		text = text.replaceAll( "  ", "&nbsp; " );
		text = text.replaceAll( "\t", "&nbsp;&nbsp;&nbsp;&nbsp; " );

		return text;
	}
}
