/*
	Copyright (C) 2003 Adam Olsen

	This program is free software; you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation; either version 1, or (at your option)
	any later version.

	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with this program; if not, write to the Free Software
	Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/

package com.valhalla.jbother;

import java.awt.*;
import java.awt.event.*;

import javax.swing.*;
import java.util.*;
import com.valhalla.gui.*;
import com.valhalla.jbother.*;
import com.valhalla.jbother.jabber.BuddyStatus;
import com.valhalla.settings.Settings;

/**
 * Handles Headline messages (server announces, RSS, etc.)
 *
 * @author Yury Soldak (tail)
 * @version 1.0
 **/
public class HeadlinesPanel extends ConversationPanel
{
	private ResourceBundle resources = ResourceBundle.getBundle( "JBotherBundle", Locale.getDefault() );

	private JScrollPane conversationScroll;
	private JPopupMenu popMenu = new JPopupMenu();


	/**
	 * Default constructor
	 * @param buddy the Buddy that this window corresponds to
	 * @param userId the user id of the buddy
	 * @param buddyName the buddy's alias
	*/
	public HeadlinesPanel( BuddyStatus buddy )
	{
		super( buddy );

		conversationScroll = new JScrollPane( conversationArea );
		conversationScroll.setVerticalScrollBarPolicy( JScrollPane.VERTICAL_SCROLLBAR_AS_NEEDED );
		JMenuItem viewLog = new JMenuItem( resources.getString( "viewLog" ) );
		popMenu.add( viewLog );

		viewLog.addActionListener( new ActionListener()
		{
			public void actionPerformed( ActionEvent e ) { openLogWindow(); }
		} );

		conversationArea.setScroll( conversationScroll );
		setLayout( new BoxLayout( this, BoxLayout.Y_AXIS ) );
		setBorder( BorderFactory.createEmptyBorder( 5, 5, 5, 5 ) );
		add( conversationScroll );
		addListeners();
	}

	public void focusYourself() { }

	/**
	**/
	public void createFrame()
	{
		frame = new JFrame();
		frame.setContentPane( this );
		frame.pack();
		frame.setSize( new Dimension( 400, 420 ) );
		ImageIcon icon = StatusIconCache.getStatusIcon( org.jivesoftware.smack.packet.Presence.Mode.AVAILABLE );
		if( icon != null )
		{
			frame.setIconImage( icon.getImage() );
		}

		Standard.cascadePlacement( frame );

		frame.addWindowListener(
			new WindowAdapter()
			{
				public void windowClosing( WindowEvent e )
				{
					if( Settings.getInstance().getProperty( "preserveMessages" ) == null )
					{
						closeHandler();
					}
					else
					{
						startTimer();
						frame.setVisible( false );
					}
				}
			} );

		frame.setTitle( buddy.getName() + " (" + buddy.getUser() + ")" );

		if( !frame.isVisible() )
		{
			frame.setVisible( true );
		}

		stopTimer();
	}

	/**
	 * Adds the various event listeners for the components that are a part of this
	 * frame
	 **/
	private void addListeners()
	{
		conversationArea.addKeyListener( new CTRLWHandler() );
		conversationArea.addMouseListener( new RightClickListener( popMenu ) );
	}

	/**
	 * Closes the Window if CTRL+W is pressed
	*/
	class CTRLWHandler extends KeyAdapter
	{
		public void keyTyped( KeyEvent e )
		{
			if( e.getKeyCode() == KeyEvent.VK_W && e.getModifiers() == Toolkit.getDefaultToolkit().getMenuShortcutKeyMask() )
			{
				checkCloseHandler();
			}

			if( e.getKeyCode() == KeyEvent.VK_K && e.getModifiers() == Toolkit.getDefaultToolkit().getMenuShortcutKeyMask() )
			{
				closeHandler();
			}
		}
	}

	/**
	 * Recieves a message
	 * @param sbj the subject
	 * @param body the message
	 * @param resource the resource of the buddy
	*/
	public void recieveMessage( String sbj, String body, String resource )
	{
		body = ConversationText.replaceText( body, true);
		String text = "<div style=\"margin:2px; background-color:#f0f0f0; padding:3px;\"><b>"+sbj+"</b><hr>"+body+"</div>";

		conversationArea.append( text );

		super.recieveMessage();

		if( logOut != null )
		{
			logOut.println( conversationArea.getLastHTML() );
		}
	}

}
