/*
 *  Copyright (C) 2003 Adam Olsen
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 1, or (at your option)
 *  any later version.
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
package com.valhalla.jbother;

import java.util.*;
import javax.swing.*;
import java.awt.*;
import com.valhalla.settings.*;

/**
 * Displays a ConversationPanel in the TabFrame or in a containing JFrame
 * depending on whether or not the application is set to use a tabbed window
 *
 * @author     Adam Olsen
 * @created    Oct 25, 2004
 * @version    1.1
 */
public class MessageDelegator
{
	private static MessageDelegator instance = null;
	private Vector panels = new Vector();

	/**
	 * Default constructor... private for singleton
	 */
	private MessageDelegator()
	{
	}

	/**
	 * @return    the MessageDelegator instance
	 */
	public static MessageDelegator getInstance()
	{
		if( instance == null )
		{
			instance = new MessageDelegator();
		}
		return instance;
	}

	/**
	 * Shows a panel using the TabFrame or a containing frame
	 *
	 * @param  panel  the panel to show
	 */
	public void showPanel( ConversationPanel panel )
	{
		if( !( panel instanceof MessagePanel ) && Settings.getInstance().getBoolean( "useTabbedWindow" ) )
		{
			BuddyList.getInstance().startTabFrame();
			if( !BuddyList.getInstance().getTabFrame().contains( panel ) )
			{
				BuddyList.getInstance().addTabPanel( panel );
				if( panel instanceof ChatPanel )
				{
					( (ChatPanel)panel ).setUpDivider();
				}
			}
		}
		else
		{
			if( panel.getContainingFrame() == null )
			{
				panel.createFrame();
			}
		}

		if( !panels.contains( panel ) )
		{
			panels.add( panel );
		}
	}

	/**
	 * If the panel is contained in a JFrame, this method brings that frame to the front
	 * of the screen
	 *
	 * @param  panel  the panel containing the frame to bring to the front
	 */
	public void frontFrame( ConversationPanel panel )
	{
		if( Settings.getInstance().getBoolean( "useTabbedWindow" ) )
		{
			JTabbedPane pane = BuddyList.getInstance().getTabFrame().getTabPane();
			pane.setSelectedComponent( (Component)panel );
			return;
		}

		JFrame frame = panel.getContainingFrame();
		if( frame != null )
		{
			frame.setVisible( true );
			frame.toFront();
		}
	}

	/**
	 * Removes a panel from the panels Vector
	 *
	 * @param  panel  the panel to remove
	 */
	public void removePanel( ConversationPanel panel )
	{
		panels.remove( panel );
	}

	/**
	 * @return    the Vector containing a list of all the available ConversationPanels
	 */
	public Vector getPanels()
	{
		return panels;
	}
}

