/*
    Copyright (C) 2003 Adam Olsen

	This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 1, or (at your option)
    any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/

package com.valhalla.jbother;

import com.valhalla.jbother.*;
import com.valhalla.gui.*;
import javax.swing.*;
import java.awt.*;
import java.awt.event.*;
import java.util.*;

import org.jivesoftware.smack.packet.Message;

/**
 * A blank message window - to send "NORMAL" type messages
 *
 * @author Adam Olsen
 * @version 1.0
*/
public class MessagePanel extends ConversationPanel implements TabFramePanel
{
	private ResourceBundle resources = ResourceBundle.getBundle( "JBotherBundle", Locale.getDefault() );
	private JLabel toLabel = new JLabel( resources.getString( "to" ) + ": " );
	private JLabel subjectLabel = new JLabel( resources.getString( "subject" ) + ": " );
	private JButton sendButton = new JButton( resources.getString( "send" ) );
	private JButton replyButton = new JButton( resources.getString( "reply" ) );
	private JTextField toField = new JTextField();
	private JTextField subjectField = new JTextField();
	private JTextArea textEntryArea = new JTextArea();
	private JPanel buttonPanel = new JPanel();
	private JScrollPane scroll = new JScrollPane();
	private MessagePanel thisPointer = this;

	/**
	 * Default constructor
	*/
	public MessagePanel()
	{
		super( null );

		setBorder( BorderFactory.createEmptyBorder( 5, 5, 5, 5 ) );
		setLayout( new BorderLayout( 5, 5 ) );

		JPanel topPanel = new JPanel();
		topPanel.setLayout( new BoxLayout( topPanel, BoxLayout.Y_AXIS ) );

		JPanel toPanel = new JPanel( new BorderLayout( 5, 5 ) );
		toPanel.add( toLabel, BorderLayout.WEST );
		toPanel.add( toField, BorderLayout.CENTER );
		toPanel.setBorder( BorderFactory.createEmptyBorder( 0, 0, 5, 0 ) );
		topPanel.add( toPanel );

		JPanel subjectPanel = new JPanel( new BorderLayout( 5, 5 ) );
		subjectPanel.add( subjectLabel, BorderLayout.WEST );
		subjectPanel.add( subjectField, BorderLayout.CENTER );
		topPanel.add( subjectPanel );

		add( topPanel, BorderLayout.NORTH );
		scroll.setViewportView( textEntryArea );
		conversationArea.setScroll( scroll );
		add( scroll, BorderLayout.CENTER );

		buttonPanel.setLayout( new BoxLayout( buttonPanel, BoxLayout.X_AXIS ) );
		buttonPanel.add( Box.createHorizontalGlue() );
		buttonPanel.add( sendButton );

		addListeners();

		add( buttonPanel, BorderLayout.SOUTH );

		toField.grabFocus();
	}

	/**
	**/
	public void createFrame()
	{
		frame = new JFrame();
		frame.setContentPane( this );

		frame.pack();
		frame.setSize( new Dimension( 450, 370 ) );

		ImageIcon icon = StatusIconCache.getStatusIcon( org.jivesoftware.smack.packet.Presence.Mode.AVAILABLE );
		if( icon != null )
		{
			frame.setIconImage( icon.getImage() );
		}

		Standard.cascadePlacement( frame );

		frame.addWindowListener(
			new WindowAdapter()
			{
				public void windowClosing( WindowEvent e )
				{
					closeHandler();
				}
			} );

		if( !frame.isVisible() )
		{
			frame.setVisible( true );
		}

	}

	/**
	 * Focuses when a Tab in the TabFrame is clicked
	*/
	public void focusYourself()
	{
		if( toField.isEnabled() )
		{
			toField.grabFocus();
		}
		else {
			conversationArea.grabFocus();
		}
	}

	/**
	 * Recieves a message and displays it in the Dialog
	 * @param message the message packet to recieve
	*/
	public void recieveMessage( Message message )
	{
		super.recieveMessage();
		toLabel.setText( resources.getString( "from" ) + ": " );
		toField.setText( message.getFrom() );
		toField.setEnabled( false ); // to and subject fields are disabled in the
									 // receiving dialog
		subjectField.setText( message.getSubject() );
		subjectField.setEnabled( false );

		scroll.setViewportView( conversationArea );
		String body = ConversationText.replaceText( message.getBody(), false );

		conversationArea.append( body );
		buttonPanel.remove( sendButton );
		buttonPanel.add( replyButton );
		buttonPanel.repaint();

		validate();
		MessageDelegator.getInstance().showPanel( this );
		MessageDelegator.getInstance().frontFrame( this );
	}

	/**
	 * @return the name of the tab
	*/
	public String getPanelName()
	{
		if( !toField.getText().equals( "" ) )
		{
			return toField.getText();
		}
		else {
			return "Blank Message";
		}
	}

	/**
	 * @return the name of the tab
	*/
	public String getWindowTitle()
	{
		if( !toField.getText().equals( "" ) )
		{
			return toField.getText();
		}
		else {
			return "Blank Message";
		}
	}

	/**
	 * Sets the text in the "To: " field
	 * @param to the string to set the text to
	*/
	public void setTo( String to )
	{
		toField.setText( to );
	}

	/**
	 * Sets the text in the "Subject:" field
	 * @param subject the string to set the subject text to
	*/
	public void setSubject( String subject )
	{
		subjectField.setText( subject );
	}

	/**
	 * Returns the TextArea widget
	 * @return The main text area
	*/
	public JTextArea getTextEntryArea() { return textEntryArea; }

	/**
	 * Called by the reply button to create a reply window
	*/
	private void replyHandler()
	{
		MessagePanel window = new MessagePanel();
		window.setTo( toField.getText() );
		window.setSubject( "Re: " + subjectField.getText() );
		window.getTextEntryArea().grabFocus();
		window.validate();
		MessageDelegator.getInstance().showPanel( window );
		closeHandler();
	}

	/**
	 * Adds the listeners to the various event emitting widgets
	*/
	private void addListeners()
	{
		final MessagePanel thisPointer = this;

		MessageActionListener listener = new MessageActionListener();
		sendButton.addActionListener( listener );
		replyButton.addActionListener( listener );

		Action closeAction = new AbstractAction()
		{
			public void actionPerformed( ActionEvent e )
			{
				closeHandler();
			}
		};

		toField.getInputMap().put( KeyStroke.getKeyStroke( KeyEvent.VK_W, Toolkit.getDefaultToolkit().getMenuShortcutKeyMask() ), closeAction );
		subjectField.getInputMap().put( KeyStroke.getKeyStroke( KeyEvent.VK_W, Toolkit.getDefaultToolkit().getMenuShortcutKeyMask() ), closeAction );
		textEntryArea.getInputMap().put( KeyStroke.getKeyStroke( KeyEvent.VK_W, Toolkit.getDefaultToolkit().getMenuShortcutKeyMask() ), closeAction );
		conversationArea.getInputMap().put( KeyStroke.getKeyStroke( KeyEvent.VK_W, Toolkit.getDefaultToolkit().getMenuShortcutKeyMask() ), closeAction );
	}

	/**
	 * Sends the message
	*/
	private void sendHandler()
	{
		if( toField.getText().equals( "" ) || textEntryArea.getText().equals( "" ) )
		{
			Standard.warningMessage( this, "messageWindow",
				resources.getString( "mustSpecifyToAndBody" ) );
			return;
		}

		Message message = new Message();

		// sets up the message
		message.setBody( textEntryArea.getText() );
		message.setType( Message.Type.NORMAL );
		message.setTo( toField.getText() );
		message.setSubject( subjectField.getText() );
		message.setFrom( BuddyList.getInstance().getConnection().getUser() );

		BuddyList.getInstance().getConnection().sendPacket( message );

		closeHandler();
	}

	/**
	 * Handles the events in the MessageDialog
	 * @author Adam Olsen
	 * @version 1.0
	*/
	class MessageActionListener implements ActionListener
	{
		/**
		 * called by the button widgets
		*/
		public void actionPerformed( ActionEvent e )
		{
			if( e.getSource() == sendButton ) sendHandler();
			if( e.getSource() == replyButton ) replyHandler();
		}
	}
}
