/*
	Copyright (C) 2003 Adam Olsen

	This program is free software; you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation; either version 1, or (at your option)
	any later version.

	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with this program; if not, write to the Free Software
	Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/

package com.valhalla.jbother;

import com.valhalla.jbother.jabber.*;
import com.valhalla.jbother.jabber.smack.*;

import java.util.*;
import org.jivesoftware.smack.*;
import org.jivesoftware.smack.packet.*;
import org.jivesoftware.smack.filter.*;
import com.valhalla.jbother.*;

/**
 * Collects version info about Jabber users as they sign in.
 * Runs in the background and collects <pre>jabber:iq:version</pre> about
 * users as they sign in
 *
 * @author Adam Olsen
 * @version 1.0
**/
public class VersionCollectorThread implements Runnable
{
	private Vector buddies = new Vector();

	/**
	 * Adds a buddy to the queue
	 * @param buddy the buddy to add to the queue
	**/
	public void add( BuddyStatus buddy )
	{
		synchronized( buddies )
		{
			buddies.add( buddy );
		}
	}

	/**
	 * To be called when the thread starts
	**/
	public void run()
	{
		while( true )
		{
			try {
				Thread.sleep( 1500 );
			}
			catch( InterruptedException e ) { }

			getVersions();
		}
	}

	/**
	 * Gets the versions of the buddies in the queue
	**/
	private void getVersions()
	{
		synchronized( buddies )
		{
			while( buddies.size() > 0 )
			{
				BuddyStatus buddy = (BuddyStatus)buddies.firstElement();
				String resource = buddy.getHighestResource();
				String user = buddy.getUser();

				if( resource != null && !resource.equals( "_no resource_"  ) ) user += "/" + resource;
				XMPPConnection con = BuddyList.getInstance().getConnection();

				Version request = new Version();
				request.setType(IQ.Type.GET);
				request.setTo( user );

				// Create a packet collector to listen for a response.
				PacketCollector collector = con.createPacketCollector(
					new PacketIDFilter( request.getPacketID() ) );

				con.sendPacket( request );

				// Wait up to 5 seconds for a result.
				IQ result = (IQ)collector.nextResult( 3000 );
				if( result != null && result.getType() == IQ.Type.RESULT )
				{
					 Version v = (Version)result;
					 buddy.setVersionInfo( v.getName() + " " + v.getVersion() + " / " + v.getOs() );
				}

				buddies.remove( buddy );
			}
		}
	}
}
