/*
	Copyright (C) 2003 Adam Olsen

	This program is free software; you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation; either version 1, or (at your option)
	any later version.

	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with this program; if not, write to the Free Software
	Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/

package com.valhalla.jbother.groupchat;

import java.awt.*;
import java.awt.event.*;
import java.beans.*;
import java.io.*;
import java.text.SimpleDateFormat;
import java.util.*;
import java.util.regex.Pattern;
import javax.swing.*;
import javax.swing.event.*;
import javax.swing.text.html.HTMLDocument;
import org.jivesoftware.smack.*;
import org.jivesoftware.smack.packet.*;
import com.valhalla.gui.*;
import com.valhalla.jbother.*;
import com.valhalla.jbother.jabber.BuddyStatus;
import com.valhalla.jbother.jabber.smack.*;
import com.valhalla.settings.Settings;

/**
 * This is the panel that contains a groupchat conversation.  It is placed in a JTabbedPane in
 * GroupChat frame.
 *
 * @author Adam Olsen
 **/
public class ChatRoomPanel extends JPanel implements LogViewerCaller, TabFramePanel
{
	private ResourceBundle resources = ResourceBundle.getBundle( "JBotherBundle", Locale.getDefault() );
	private JTextField textEntryArea = new JTextField( 60 );

	private StringBuffer conversationText = new StringBuffer();
	private JScrollPane conversationScroll;
	private ConversationArea conversationArea = new ConversationArea();

	private JMenuItem
						clearItem = new JMenuItem( resources.getString( "clear" ) ),
						logItem = new JMenuItem( resources.getString( "viewLog" ) ),
						newItem = new JMenuItem( resources.getString( "joinRoom" ) ),
						leaveItem = new JMenuItem( resources.getString( "leaveRoom" ) ),
						topicItem = new JMenuItem( resources.getString( "setRoomSubject" ) ),
						nickItem = new JMenuItem( resources.getString( "changeNickname" ) );

	private JButton topicButton = new JButton( resources.getString( "changeButton" ) );
	private JPopupMenu popMenu = new JPopupMenu();

	private int oldMaximum = 0;
	private HTMLDocument document = (HTMLDocument)conversationArea.getDocument();
	private JPanel scrollPanel = new JPanel( new GridLayout( 1, 0 ) );
	private JSplitPane mainPanel = new JSplitPane( JSplitPane.HORIZONTAL_SPLIT );
	private JSplitPane container;
	private GroupChat chat;
	private String chatroom, nickname;
	private Hashtable buddyStatuses = new Hashtable();
	private GroupChatNickList nickList;
	private WaitDialog wait = new WaitDialog( resources.getString( "pleaseWait" ), resources.getString( "attemptingToJoin" ) );
	private String subject = resources.getString( "noSubject" );
	private JLabel subjectLabel = new JLabel();
	private int countOfNewMessages = 0;
	// for logging
	private File logFile;
	private PrintWriter logOut;
	private FileWriter fw;
	private String currentNick;
	private ChatRoomPanel thisPointer = this;
	private boolean listenersAdded = false;

	/**
	 * This sets up the appearance of the chatroom window
	 * @param chatroom the chatroom address
	 * @param nickname the nickname to use when joining
	 **/
	public ChatRoomPanel( String chatroom, String nickname )
	{
		this.chatroom = chatroom;
		this.nickname = nickname;

		BuddyList.getInstance().startTabFrame();

		setLayout( new BoxLayout( this, BoxLayout.Y_AXIS ) );
		setBorder( BorderFactory.createEmptyBorder( 5, 5, 5, 5 ) );

		JPanel subjectPanel = new JPanel();
		subjectPanel.setLayout( new BoxLayout( subjectPanel, BoxLayout.Y_AXIS ) );
		subjectPanel.add( subjectLabel );
		subjectPanel.add( Box.createHorizontalGlue() );
		add( subjectPanel );
		add( Box.createRigidArea( new Dimension( 0, 5 ) ) );

		add( mainPanel );

		nickList = new GroupChatNickList( this );

		String divLocString = Settings.getInstance().getProperty( "chatWindowDividerLocation" );
		int divLoc = 0;

		if( divLocString != null )
		{
			divLoc = Integer.parseInt( divLocString );
		}
		else {
			Dimension dimension = BuddyList.getInstance().getTabFrame().getSize();
			divLoc = (int)dimension.getWidth() - 127;
		}

		mainPanel.setDividerLocation( divLoc );
		mainPanel.setOneTouchExpandable( true );
		mainPanel.setResizeWeight( 1 );
		mainPanel.addPropertyChangeListener( "lastDividerLocation", new DividerListener() );

		conversationScroll = new JScrollPane( conversationArea );
		conversationScroll.setVerticalScrollBarPolicy( JScrollPane.VERTICAL_SCROLLBAR_ALWAYS );
		conversationArea.setScroll( conversationScroll );

		setUpPopMenu();

		scrollPanel.add( conversationScroll );

		JPanel containerPanel = new JPanel();
		GridBagLayout grid = new GridBagLayout();
		GridBagConstraints c = new GridBagConstraints();

		c.gridx = 0;
		c.gridy = 0;
		c.fill = GridBagConstraints.BOTH;
		c.weightx = 1;
		c.weighty = 1;

		c.gridwidth = 1;

		grid.setConstraints( scrollPanel, c );
		containerPanel.setLayout( grid );
		containerPanel.add( scrollPanel );

		c.gridy++;
		c.weightx = 1;
		c.weighty = 0;
		c.fill = GridBagConstraints.HORIZONTAL;

		Component sep = Box.createRigidArea( new Dimension( 0, 5 ) );
		grid.setConstraints( sep, c );
		containerPanel.add( sep );

		c.gridy++;

		textEntryArea.setBorder( BorderFactory.createEtchedBorder() );
		grid.setConstraints( textEntryArea, c );

		containerPanel.add( textEntryArea );

		mainPanel.add( containerPanel );
		mainPanel.add( nickList );

		textEntryArea.grabFocus();
        textEntryArea.setFocusTraversalKeysEnabled(false); //for disable focus traversal with TAB
		setSubject( subject );

		addListeners();
	}

	/**
	 * @return true if the TabFrame panel listeners have already been added to this panel
	*/
	public boolean listenersAdded() { return listenersAdded; }

	/**
	 * Sets whether or not the TabFrame panel listeners have been added
	 * @param added true if they have been added
	*/
	public void setListenersAdded( boolean added ) { this.listenersAdded = added; }

	/**
	 * @return the input area of this panel
	*/
	public Component getInputComponent() { return textEntryArea; }

	/**
	 * @return the JList representing the nicklist
	*/
	public JList getNickList() { return nickList.getList(); }

	/**
	 * @return the text entry area
	*/
	public JTextField getTextEntryArea() { return textEntryArea; }

	/**
	 * Listens for a change in the divider location, and saves it for later retreival
	 * @author Adam Olsen
	 * @version 1.0
	*/
	private class DividerListener implements PropertyChangeListener
	{
		public void propertyChange( PropertyChangeEvent e )
		{
			Settings.getInstance().setProperty( "chatWindowDividerLocation", e.getOldValue().toString() );
			BuddyList.getInstance().getTabFrame().saveStates();
		}
	}

	/**
	 * Updates the font in the ConversationArea
	 * @param font the font to update to
	*/
	public void updateStyle( Font font ) { conversationArea.loadStyleSheet( font ); }

	/**
	 * Look for a right click, and show a pop up menu
	 * @author Adam Olsen
	 * @version 1.0
	**/
	class RightClickListener extends MouseAdapter
	{
		public void mousePressed( MouseEvent e ) { checkPop( e ); }
		public void mouseReleased( MouseEvent e ) { checkPop( e ); }
		public void mouseClicked( MouseEvent e ) { checkPop( e ); }

		public void checkPop( MouseEvent e )
		{
			// look for the popup trigger.. usually a right click
			if( e.isPopupTrigger() )
			{
				popMenu.show( e.getComponent(), e.getX(), e.getY() );
			}
		}
	}

	/**
	 * Add the various menu items to the popup menu
	**/
	private void setUpPopMenu()
	{
		MenuItemListener listener = new MenuItemListener();

		conversationArea.addMouseListener( new RightClickListener() );

		popMenu.add( clearItem );
		popMenu.add( topicItem );
		popMenu.add( nickItem );
		popMenu.addSeparator();
		popMenu.add( logItem );

		popMenu.addSeparator();
		popMenu.add( newItem );
		popMenu.add( leaveItem );

		topicButton.addActionListener( listener );
		clearItem.addActionListener( listener );
		logItem.addActionListener( listener );
		newItem.addActionListener( listener );
		leaveItem.addActionListener( listener );
		topicItem.addActionListener( listener );
		nickItem.addActionListener( listener );
	}

	/**
	 * Listens for items to be selected in the menu
	 * @author Adam Olsen
	 * @version 1.0
	*/
	private class MenuItemListener implements ActionListener
	{
		public void actionPerformed( ActionEvent e )
		{
			if( e.getSource() == nickItem ) changeNickHandler();
			if( e.getSource() == leaveItem )
			{
				BuddyList.getInstance().getTabFrame().removePanel( thisPointer );
				BuddyList.getInstance().stopTabFrame();
			}
			if( e.getSource() == topicItem || e.getSource() == topicButton ) topicHandler();
			if( e.getSource() == clearItem ) conversationArea.setText( "" );
			if( e.getSource() == newItem )
			{
				GroupChatBookmarks gc = new GroupChatBookmarks();
				gc.show();
				gc.toFront();
			}

			if( e.getSource() == logItem ) new LogViewerDialog( thisPointer, getRoomName() );
		}
	}

	/**
	 * Opens the log window for this chat room
	*/
	public void openLogWindow()
	{
		new LogViewerDialog( this, getRoomName() );
	}

	/**
	 * Adds a buddy to the nickname list
	 * @param buddy the buddy to add
	*/
	public void addBuddy( String buddy ) { nickList.addBuddy( buddy ); }

	/**
	 * Removes a buddy from the nick list
	 * @param buddy the buddy to remove
	*/
	public void removeBuddy( String buddy )
	{
		nickList.removeBuddy( buddy );
		buddyStatuses.remove( buddy );
	}

	/**
	 * Opens the log file
	*/
	public void startLog()
	{
		// for loggingg
		if( Settings.getInstance().getBoolean( "keepLogs" ) )
		{
			try {
				String logFileName = LogViewerDialog.getDateName() + ".log.html";
				String logFileDir = JBother.profileDir + File.separatorChar +
					"logs" + File.separatorChar + getRoomName().replaceAll( "@", "_at_" ).replaceAll( "\\/", "-" );

				File logDir = new File( logFileDir );

				if( !logDir.isDirectory() && !logDir.mkdirs() ) Standard.warningMessage( this, resources.getString( "log" ),
					resources.getString( "couldNotCreateLogDir" ) );
				logFile = new File( logDir, logFileName );
				fw = new FileWriter( logFile, true );
				logOut = new PrintWriter( fw, true );
			}
			catch( IOException e ) { com.valhalla.Logger.debug( "Could not open logfile." ); }
		}
	}

	/**
	 * Gets the BuddyStatus represending a user in the room
	 * @param user the BuddyStatus to get
	 * @return the requested BuddyStatus
	*/
	public BuddyStatus getBuddyStatus( String user )
	{
		if( !buddyStatuses.containsKey( user ) )
		{
			BuddyStatus buddy = new BuddyStatus( user );
			buddy.setName( user.substring( user.indexOf( "/" ) + 1, user.length() ) );
			buddyStatuses.put( user, buddy );
		}

		return (BuddyStatus)buddyStatuses.get( user );
	}

	/**
	 * Returns the tab name for the TabFramePanel
	 * @return the panel name
	*/
	public String getPanelName() { return getShortRoomName(); }

	/**
	 * Returns the tooltip for the tab in the TabFrame
	 * @return the tooltip for this tab in the tab frame
	*/
	public String getTooltip() { return getRoomName(); }

	/**
	 * Returns the window title
	 * @return the window title for the TabFrame when this tab is selected
	*/
	public String getWindowTitle() { return resources.getString( "groupChat" ) + ": " + getRoomName(); }

	/**
	 * Gets the short room name - for example, if you are talking in
	 * jdev@conference.jabber.org, it would return "jdev"
	 * @return short room name
	 **/
	public String getShortRoomName()
	{
		return chat.getRoom().replaceAll( "\\@.*", "" );
	}

	/**
	 * Gets the entire room name, server included
	 * @return gets the room address
	 **/
	public String getRoomName()
	{
		return chat.getRoom();
	}

	/**
	 * Starts the groupchat.  Sets up a thread to connect, and start that thread
	 **/
	public void startChat()
	{
		if( !BuddyList.getInstance().checkConnection() ) return;

		chat = new GroupChat( BuddyList.getInstance().getConnection(), chatroom );

		//set up the presence and message listeners so that they are
		//ready from the momemnt we start the chat.
		chat.addMessageListener( new GroupChatMessagePacketListener( this ) );
		chat.addParticipantListener( new GroupParticipantListener( this ) );
		wait.setVisible( true );

		Thread thread = new Thread( new JoinChatThread() );
		thread.start();

		startLog();
	}

	/**
	 * Asks for a new topic, then sets the new topic
	*/
	private void topicHandler()
	{
		//find out what they want the subject to be
		String result = (String)JOptionPane.showInputDialog( null,
			resources.getString( "enterSubject" ),
			resources.getString( "setSubject" ), JOptionPane.QUESTION_MESSAGE, null, null, subject );

		if( result != null && !result.equals( "" ) )
		{
			//send a packet to the server with a subject and an empty body
			Message packet = new Message( getRoomName(), Message.Type.GROUP_CHAT );
			packet.setSubject( result );
			packet.setBody( "/me " + resources.getString( "hasSetTopic" ) + " " + result );

			if( !BuddyList.getInstance().checkConnection() )
			{
				BuddyList.getInstance().connectionError();
				return;
			}
			else {
				BuddyList.getInstance().getConnection().sendPacket( packet );
			}

			setSubject( result );
		}
	}

	/**
	 * Asks for a new nickname, and sends a nickname change request
	*/
	private void changeNickHandler()
	{
		if( currentNick == null ) currentNick = chat.getNickname();
		String result = (String)JOptionPane.showInputDialog( null,
			resources.getString( "enterNickname" ),
			resources.getString( "setNickname" ), JOptionPane.QUESTION_MESSAGE, null, null, currentNick );

		if( result != null && !result.equals( "" ) )
		{
			//send a packet to the server with a subject and an empty body
			Presence packet = new Presence( Presence.Type.AVAILABLE, BuddyList.getInstance().getCurrentStatusString(), 0, BuddyList.getInstance().getCurrentPresenceMode() );
			packet.setTo( getRoomName() + '/' + result );
			packet.setFrom( getRoomName() + '/' + currentNick );

			if( !BuddyList.getInstance().checkConnection() )
			{
				BuddyList.getInstance().connectionError();
				return;
			}
			else {
				BuddyList.getInstance().getConnection().sendPacket( packet );
			}

			currentNick = result;
		}
	}


	/**
	 * Adds the event listeners for the various components in this chatwindows
	 **/
	public void addListeners()
	{
		//set up the window so you can press enter in the text box and
		//that will send the message.
		Action SendMessageAction = new AbstractAction()
		{
			public void actionPerformed( ActionEvent e )
			{
				sendHandler();
			}
		};

		Action nickCompletionAction = new AbstractAction()
		{
			public void actionPerformed( ActionEvent e )
			{
				nickCompletionHandler();
			}
		};

		//set it up so that if they drag in the conversation window, it grabs the focus
		conversationArea.addMouseMotionListener( new MouseMotionAdapter()
		{
			public void mouseDragged( MouseEvent e )
			{
				conversationArea.grabFocus();
			}
		} );

		//set it up so that if there isn't any selected text in the conversation area
		//the textentryarea grabs the focus.
		conversationArea.addFocusListener( new FocusListener()
		{
			public void focusGained( FocusEvent e )
			{
				if( conversationArea.getSelectedText() == null )
				{
					textEntryArea.requestFocus();
				}
				else conversationArea.grabFocus();
			}

			public void focusLost( FocusEvent e ) { }
		} );

		Action closeAction = new AbstractAction()
		{
			public void actionPerformed( ActionEvent e )
			{
				BuddyList.getInstance().getTabFrame().removePanel( thisPointer );
				BuddyList.getInstance().stopTabFrame();
			}
		};

		KeyStroke enterStroke = KeyStroke.getKeyStroke( KeyEvent.VK_ENTER, 0 );
		textEntryArea.getInputMap().put( enterStroke, SendMessageAction );

		KeyStroke tabStroke = KeyStroke.getKeyStroke( KeyEvent.VK_TAB, 0 );
		textEntryArea.getInputMap().put( tabStroke, nickCompletionAction );

		textEntryArea.getInputMap().put( KeyStroke.getKeyStroke( KeyEvent.VK_W, Toolkit.getDefaultToolkit().getMenuShortcutKeyMask() ), closeAction );
	}

	/**
	 * Leaves this room and removes it from the groupchat frame
	*/
	public void leave()
	{
		closeLog();
		try {
			chat.leave();
		}
		catch( IllegalStateException e ) { }
	}	//leave the chatroom

	/**
	 * Focuses the text entry area
	*/
	public void focusYourself()
	{
		conversationArea.moveToEnd();
		textEntryArea.grabFocus();

		countOfNewMessages = 0;
		JTabbedPane tabPane = BuddyList.getInstance().getTabFrame().getTabPane();
		tabPane.setTitleAt(tabPane.indexOfComponent(this),this.getShortRoomName());
	}

	/**
	 * Gets the nickname currently being used in the chat room
	 * @return the nickname being used in the chatroom
	*/
	public String getNickname()
	{
		if( currentNick == null ) currentNick = chat.getNickname();
		return currentNick;
	}

	/**
	 * Recieves a message
	 * @param from who it's from
	 * @param message the message
	*/
	public void recieveMessage( String from, String message )
	{
		message = ConversationText.replaceText( message, true ); //replace emoticons, links, etc...
		boolean messageToMe = false;
		if( from.equals( "" ) || from.toLowerCase().equals( chat.getRoom().toLowerCase() ) )
		{
			//server message
			conversationArea.append( "<div>"+getDate() + "<font color='green'> -> " + message + "</font></div>" );
		}
		else {

			boolean highLightedSound = false;

			if( message.startsWith( " /me " ) )
			{
				message = message.replaceAll( "^ \\/me ", "" );
				conversationArea.append( "<div>"+getDate() +
					" <b><font color='maroon'>* " + from + "</font></b> " + message + "</div>" );
			}
			else if( message.replaceAll( "<[^>]*>", "" ).matches( ".*(^|\\W)" + nickname + "\\W.*" ) )
			{
				messageToMe = true;
				conversationArea.append( "<div style='background-color: #FFC4C4;'>"+getDate() + " <b><font color='#16569e'>" + from + "</font></b>: " + message + "</div>" );
				com.valhalla.jbother.sound.SoundPlayer.play( "groupHighlightedSound" );
				highLightedSound = true;
			}
			else {
				conversationArea.append( "<div>"+getDate() + " <font color='#16569e'><b>" +
					from + "</b></font>: " + message + "</div>" );
			}

			if( !highLightedSound ) com.valhalla.jbother.sound.SoundPlayer.play( "groupRecievedSound" );
		}

		try {
			JTabbedPane tabPane = BuddyList.getInstance().getTabFrame().getTabPane();
			if (tabPane.getSelectedComponent() != this) {
				countOfNewMessages++;
				String title = this.getShortRoomName()+" ("+countOfNewMessages+")";
				if (messageToMe) title = "*"+title;
				tabPane.setTitleAt(tabPane.indexOfComponent(this),title);
			}
		}
		catch( Exception ex ) { }

		if( logOut != null )
		{
			logOut.println( conversationArea.getLastHTML() );
		}
	}

	/**
	 * Closes the log file
	*/
	public void closeLog()
	{
		if( fw != null )
		{
			try {
				fw.close();
			}
			catch( IOException e ) { }
		}
	}

	/**
	 * @return a String representing the current time the format: [Hour:Minute:Second]
	*/
	public String getDate()
	{
		SimpleDateFormat formatter = new SimpleDateFormat( "[HH:mm:ss]" );
		String date = formatter.format( new Date() );
		return date;
	}

	/**
	 * Sets the subject of the room
	 * @param subject the subject to set
	*/
	public void setSubject( String subject )
	{
		this.subject = subject;
		if( BuddyList.getInstance().getTabFrame() != null )
			BuddyList.getInstance().getTabFrame().setSubject( this );

		subjectLabel.setText( "<html><b>" + resources.getString( "subject" ) + ": </b>" + subject + "</html>" );
	}

	/**
	 * Returns the current room subject
	 * @return the current room subject
	*/
	public String getSubject() { return this.subject; }

	/**
	 * Gets all the buddy statuses in the room
	 * @return all BuddyStatuses
	*/
	public Hashtable getBuddyStatuses() { return this.buddyStatuses; }

	/**
	 * Sends the message currently in the textentryarea
	*/
	private void sendHandler()
	{
		String text = textEntryArea.getText();

		Message message = new Message();
		message.setTo( getRoomName() );
		message.setBody( text );
		message.setType( Message.Type.GROUP_CHAT );

		if( !textEntryArea.getText().equals( "" ) )
		{
			try {
				chat.sendMessage( message );
			}
			catch( XMPPException e )
			{
				com.valhalla.Logger.debug( "Could not send message." );
			}

			textEntryArea.setText( "" );
		}
	}

	/**
	 * Implementation of Tab nick completion in the textEntryArea
	*/
	private void nickCompletionHandler()
	{
		String text = textEntryArea.getText();

		/* if we have nothing => do nothing */
		if( !text.equals( "" ) ) {
			int caretPosition   = textEntryArea.getCaretPosition();
			int startPosition   = text.lastIndexOf(" ", caretPosition - 1) + 1;
			String nickPart     = text.substring(startPosition, caretPosition);
			Vector matches      = new Vector();

			java.util.List keys = new ArrayList( buddyStatuses.keySet() );
			Iterator iterator   = keys.iterator();

			while( iterator.hasNext() ) {
				BuddyStatus buddy = (BuddyStatus)buddyStatuses.get(iterator.next());
				try {
					String nick = buddy.getUser().substring(buddy.getUser().lastIndexOf("/")+1);
					if (nick.toLowerCase().startsWith(nickPart.toLowerCase())) {
						matches.add(nick);
					}
				} catch(java.lang.NullPointerException e) {
				}
			}

			if (matches.size() > 0) {
				String append = "";

				if (matches.size() > 1) {
					String nickPartNew = (String)matches.firstElement();
					String nick = "";
					String hint   = nickPartNew + ", ";
					int nickPartLen = nickPart.length();
					for( int i = 1; i < matches.size(); i++ ){
						nick = (String)matches.get(i);
						hint += nick + ", ";
						for( int j = 1; j <= nick.length()-nickPartLen; j++ ){
							if (!nickPartNew.regionMatches(true,nickPartLen,nick,nickPartLen,j)){
								nickPartNew = nickPartNew.substring(0, nickPartLen+j-1);
								break;
							}
						}
					}
					if (nickPart.length() != nickPartNew.length()) {
						nickPart = nickPartNew;
					}
					// emphasize differense in matches by bold and append hint to the conversationArea
					// hint = hint.replaceAll() can't be used here because of its case sensitive nature
					Pattern pattern = Pattern.compile("(" + nickPart + ")([^,]+), ", Pattern.CASE_INSENSITIVE);
					hint = pattern.matcher(hint).replaceAll("$1<b>$2</b>, ");
					conversationArea.append( "<div style=\"color: #16569e;\">" + hint.substring(0,hint.length()-2) + "</div>" );
				} else {
					nickPart = (String)matches.firstElement();
					if( startPosition == 0 ) append = ": ";
					else                     append = " ";
				}

				String newText = text.substring( 0, startPosition );
				newText += nickPart + append;
				newText += text.substring(caretPosition);
				textEntryArea.setText( newText );
				/* Set caret to the appropriate position */
				textEntryArea.setCaretPosition(startPosition + nickPart.length() + append.length());
			}

		} /* end of the lazy "if" */
	}

	/**
	 * Joins the chatroom and adds this chatroomwindow to the TabFrame
	 * @author Adam Olsen
	 * @version 1.0
	*/
	class JoinChatThread implements Runnable
	{
		private String errorMessage;

		public void run()
		{
			try {
				chat.join( nickname, 30500 );
			}
			catch( XMPPException e )
			{
				if( e.getXMPPError() != null ) errorMessage = e.getXMPPError().getMessage();
				else errorMessage = e.getMessage();
				if( errorMessage == null ) errorMessage = resources.getString( "unknownError" );
			}

			SwingUtilities.invokeLater( new Runnable()
			{
				public void run()
				{
					wait.dispose();
					if( errorMessage != null )
					{
						Standard.warningMessage( thisPointer, resources.getString( "joinChat" ), errorMessage );
						chat.leave();
						new GroupChatBookmarks().setVisible( true );
						BuddyList.getInstance().stopTabFrame();
					}
					else {
						BuddyList.getInstance().addTabPanel( thisPointer );
					}
				}
			} );
		}
	}
}
