/*
	Copyright (C) 2003 Adam Olsen

	This program is free software; you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation; either version 1, or (at your option)
	any later version.

	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with this program; if not, write to the Free Software
	Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/

package com.valhalla.jbother.jabber.smack;

import org.jivesoftware.smack.PacketListener;
import org.jivesoftware.smack.packet.Message;
import org.jivesoftware.smack.packet.Packet;

import com.valhalla.jbother.*;
import com.valhalla.jbother.groupchat.*;
import javax.swing.SwingUtilities;

/**
 * Listens for Group Chat Messages
 *
 * @author Adam Olsen
 * @version 1.0
*/
public class GroupChatMessagePacketListener implements PacketListener
{
	private ChatRoomPanel window;

	/**
	 * Sets up the group chat message listener
	 * @param window the window that this litener belongs to
	*/
	public GroupChatMessagePacketListener( ChatRoomPanel window )
	{
		this.window = window;
	}

	/**
	 * Processes the packet
	*/
	public void processPacket( Packet message )
	{
		final Message packet = (Message)message;

		if( packet.getType() != Message.Type.GROUP_CHAT ) return;

		final String from = packet.getFrom();
		if( from != null )
		{
			SwingUtilities.invokeLater( new Runnable()
			{
				public void run()
				{
					final String messageBody = packet.getBody();

					if( packet.getSubject() != null )
						window.setSubject( packet.getSubject() );

					if( messageBody != null ) window.recieveMessage( from.replaceAll( ".*\\/", "" ), messageBody );
				}
			} );
		}
	}
}
