/*
    Copyright (C) 2003 Adam Olsen

	This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 1, or (at your option)
    any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/

package com.valhalla.jbother.jabber.smack;

import java.util.TreeMap;

import org.jivesoftware.smack.PacketListener;
import org.jivesoftware.smack.packet.Packet;
import org.jivesoftware.smack.packet.Presence;

import com.valhalla.jbother.*;
import com.valhalla.jbother.jabber.*;
import com.valhalla.jbother.groupchat.*;

/**
 * Listens for presence packets when you are in a groupchat, and will update
 * the nicklist in a groupchat room.  Also, if you are in a private conversation with
 * someone and they sign off, it will let you know.
 *
 * @author Adam Olsen
 * @version 1.0
 **/
public class GroupParticipantListener implements PacketListener
{
	private ChatRoomPanel window;
	private VersionCollectorThread vThread = null;

	/**
	 * sets up the packet listener
	 */
	public GroupParticipantListener( ChatRoomPanel window )
	{
		this.window = window;
	}

	/**
	 * Processes incoming presence packets (from group chats)
	 */
	public void processPacket( Packet packet )
	{
		Presence presence = (Presence)packet;

		String from = packet.getFrom();

		boolean addBuddy = false;
		if( !window.getBuddyStatuses().containsKey( from ) ) addBuddy = true;

		//right now we find out if we have already recieved a packet from them,
		//and if not, we set up an information "account" in the system
		BuddyStatus buddyStatus = window.getBuddyStatus( from );

		//update the relavent presence information
		if( presence.getType() == Presence.Type.UNAVAILABLE )
		{
			ConversationPanel conv = buddyStatus.getConversation();
			if( (conv != null) && (conv instanceof ChatPanel) ) ((ChatPanel)conv).signedOff();
		}
		else {
			buddyStatus.addResource( "_no resource_", 5, presence.getMode(), presence.getStatus() );
			/*if( vThread == null )
			{
				vThread = new VersionCollectorThread();
				Thread thread = new Thread( vThread );
				thread.start();
			}

			vThread.add( buddyStatus );*/
		}

		//if we need to, reload the nicklist.
		if( presence.getType() == Presence.Type.AVAILABLE && addBuddy ) window.addBuddy( buddyStatus.getUser() );
		if( presence.getType() == Presence.Type.UNAVAILABLE ) window.removeBuddy( buddyStatus.getUser() );
		window.getNickList().repaint();
	}
}
