/*
    Copyright (C) 2003 Adam Olsen

	This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 1, or (at your option)
    any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/

package com.valhalla.jbother.jabber.smack;

import org.jivesoftware.smack.packet.IQ;

import java.util.*;

/**
 * IQ packet for representing jabber:iq:version
 *
 * @author Adam Olsen
 * @version 1.0
*/
public class Version extends IQ
{
	private String name = null;
	private String os = null;
	private String version = null;

	/**
	 * Default constructor - sets up the packet with no
	 * information
	*/
    public Version()
	{
        this( "", "", "" );
    }

	/**
	 * Sets up the packet with information
	 * @param name the client name
	 * @param version the client version
	 * @param os the client os
	*/
    public Version( String name, String version, String os )
	{
	    this.name = name;
	    this.version = version;
	    this.os = os;
    }

	/**
	 * @return the client name
	*/
    public String getName() { return name; }

	/**
	 * @return the client version
	*/
    public String getVersion() { return version; }

	/**
	 * @return the client os
	*/
    public String getOs() { return os; }

	/**
	 * @param name the client name
	*/
    public void setName( String name ) { this.name = name; }

	/**
	 * @param version the client version
	*/
    public void setVersion( String version ) { this.version = version; }

	/**
	 * @param os the client operating system
	*/
    public void setOs( String os ) { this.os = os; }

	/**
	 * Builds the packet
	 * @return the XML version of the packet
	*/
    public String getChildElementXML()
	{
        StringBuffer buf = new StringBuffer();
        buf.append("<query xmlns=\"jabber:iq:version\">");

		if( name != null )
		{
			buf.append( "<name>" ).append(name).append("</name>");
		}
		if( version != null )
		{
			buf.append("<version>").append(version).append("</version>");
		}
		if( os != null )
		{
			buf.append("<os>").append(os).append("</os>");
		}

        buf.append("</query>");
        return buf.toString();
    }
}
