/*
    Copyright (C) 2003 Adam Olsen
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 1, or (at your option)
    any later version.
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.
    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  */
package com.valhalla.jbother.jabber.smack.provider;

import org.jivesoftware.smack.provider.IQProvider;
import org.jivesoftware.smack.packet.*;
import org.jivesoftware.smackx.packet.Time;
import org.xmlpull.v1.XmlPullParser;

/**
 * Parses jabber:iq:time IQ packets
 *
 * @author     synic
 * @created    October 30, 2004
 */
public class TimeProvider implements IQProvider
{
	/**
	 * Parses the IQ XML
	 *
	 * @param  parser         the xml parser
	 * @return                the IQ packet
	 * @exception  Exception  if an error occurs while parsing the XML
	 */
	public IQ parseIQ( XmlPullParser parser )
		throws Exception
	{
		Time t = new Time();
		boolean done = false;

		while( !done )
		{
			int eventType = parser.next();
			if( eventType == XmlPullParser.START_TAG )
			{
				if( parser.getName().equals( "utc" ) )
				{
					t.setUtc( parser.nextText() );
				}
				else if( parser.getName().equals( "tz" ) )
				{
					t.setTz( parser.nextText() );
				}
				else if( parser.getName().equals( "display" ) )
				{
					t.setDisplay( parser.nextText() );
				}

			}
			else if( eventType == XmlPullParser.END_TAG )
			{
				if( parser.getName().equals( "query" ) )
				{
					done = true;
				}
			}
		}

		return t;
	}
}

