/*
	Copyright (C) 2003 Adam Olsen

	This program is free software; you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation; either version 1, or (at your option)
	any later version.

	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with this program; if not, write to the Free Software
	Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/

package com.valhalla.jbother.menus;

import java.awt.event.*;
import java.awt.*;
import javax.swing.*;
import java.util.*;

import com.valhalla.gui.*;
import com.valhalla.jbother.*;
import com.valhalla.settings.*;
import com.valhalla.jbother.groupchat.*;

/**
 * Buddies menu - contains all the other menus - excluding the help menu
 * @author Adam Olsen
 * @version 1.0
*/
public class BuddyListBuddiesMenu extends JMenu
{
	private BuddyList blist;
	private ResourceBundle resources = ResourceBundle.getBundle( "JBotherBundle", Locale.getDefault() );
	private JMenuItem addBuddyItem = new JMenuItem( resources.getString( "addBuddy" ) );
	private JMenuItem quitItem = new JMenuItem( resources.getString( "quitButton" ) );
	private JMenu servicesMenu = new JMenu( resources.getString( "jabberServices" ) );
	private JMenuItem joinChatItem = new JMenuItem( resources.getString( "joinGroupChat" ) );
	private JMenuItem discoItem = new JMenuItem( resources.getString( "serviceDiscovery" ) );
	private JMenuItem blankItem = new JMenuItem( resources.getString( "blankMessage" ) );
	private JCheckBoxMenuItem offlineBuddies = new JCheckBoxMenuItem( resources.getString( "showOfflineBuddies" ) );

	private JMenuItem registerItem = new JMenuItem( resources.getString( "registerForService" ) );
	private BuddyListOptionsMenu optionsMenu = new BuddyListOptionsMenu();
	private JMenuItem switchItem = new JMenuItem( resources.getString( "switchProfile" ) );
	private JMenuItem editItem = new JMenuItem( resources.getString( "editAccount" ) );

	/**
	 * Creates the buddies menu
	 * @param blist the buddy list to attach this menu to
	*/
	public BuddyListBuddiesMenu( BuddyList blist )
	{
		super( "JBother" );

		if( System.getProperty( "mrj.version" ) != null )
		{
			setText( resources.getString( "actions" ) );
			setMnemonic( KeyEvent.VK_A );
		}
		else {
			setMnemonic( KeyEvent.VK_J );
		}

		this.blist = blist;

		initComponents();
	}

	/**
	 * Sets up the visual components
	*/
	private void initComponents()
	{
		joinChatItem.addActionListener( new MenuActionListener() );

		MenuActionListener listener = new MenuActionListener();
		addBuddyItem.addActionListener( listener );
		blankItem.addActionListener( listener );
		quitItem.addActionListener( listener );
		registerItem.addActionListener( listener );
		discoItem.addActionListener( listener );
		switchItem.addActionListener( listener );
		offlineBuddies.addActionListener( listener );
		editItem.addActionListener( listener );
		offlineBuddies.setState( Settings.getInstance().getBoolean( "showOfflineBuddies" ) );

		add( addBuddyItem );

		addSeparator();
		servicesMenu.add( discoItem );
		add( servicesMenu );
		servicesMenu.add( registerItem );
		add( joinChatItem );
		add( blankItem );
		addSeparator();
		add( switchItem );
		add( editItem );
		add( offlineBuddies );
		add( optionsMenu );

		addSeparator();
		add( quitItem );
	}

	/**
	 * @param var true if this is OS X
	*/
	public void setOSX( boolean var )
	{
		if( var )
		{
			remove( quitItem );
		}
		else {
			if( !isMenuComponent( quitItem ) ) add( quitItem );
		}

		optionsMenu.setOSX( var );
	}

	/**
	 * Listens for an item to be clicked
	 * @author Adam Olsen
	 * @version 1.0
	*/
	class MenuActionListener implements ActionListener
	{
		public void actionPerformed( ActionEvent e )
		{
			if( e.getSource() == offlineBuddies ) BuddyList.getInstance().getBuddyListTree().setShowOfflineBuddies( offlineBuddies.getState() );
			else if( e.getSource() == quitItem ) blist.quitHandler();
			else if( e.getSource() == addBuddyItem )
			{
				if( !BuddyList.getInstance().checkConnection() )
				{
					BuddyList.getInstance().connectionError();
					return;
				}

				new AddBuddyDialog().show();
			}
			else if( e.getSource() == blankItem ) MessageDelegator.getInstance().showPanel( new MessagePanel() );
			else if( e.getSource() == joinChatItem ) new GroupChatBookmarks().show();
			else if( e.getSource() == registerItem ) registrationHandler();
			else if( e.getSource() == discoItem ) new ServiceDiscoveryDialog( blist ).show();
			else if( e.getSource() == switchItem ) switchHandler();
			else if( e.getSource() == editItem ) editHandler();
		}
	}

	private void editHandler()
	{
		ProfileEditorDialog dialog = new ProfileEditorDialog( null, ProfileManager.getCurrentProfile() );
		dialog.setIsCurrentProfile( true );
		dialog.show();
	}

	private void switchHandler()
	{
		if( BuddyList.getInstance().checkConnection() )
		{
			Standard.warningMessage( null, resources.getString( "error" ), resources.getString( "stillConnected" ) );
		}
		else {
			BuddyList.getInstance().setVisible( false );
			new ProfileManager().show();
		}
	}

	/**
	 * Registers for a server by displaying a RegistrationForm
	*/
	private void registrationHandler()
	{
		String result = (String)JOptionPane.showInputDialog( null,
			resources.getString( "pleaseEnterServer" ),
			resources.getString( "registerForService" ), JOptionPane.QUESTION_MESSAGE, null, null, "" );

		if( result != null && !result.equals( "" ) )
		{
			RegistrationForm form = new RegistrationForm( result );
			form.getRegistrationInfo();
		}
	}
}
