/*
    Copyright (C) 2003 Adam Olsen

	This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 1, or (at your option)
    any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/

package com.valhalla.jbother.menus;

import java.awt.event.*;
import javax.swing.*;

import org.jivesoftware.smack.*;

import com.valhalla.gui.*;
import com.valhalla.jbother.*;
import com.valhalla.pluginmanager.*;
import com.valhalla.jbother.preferences.PreferencesDialog;
import com.valhalla.settings.*;

import org.jivesoftware.smack.*;
import org.jivesoftware.smackx.*;
import org.jivesoftware.smackx.packet.*;

import java.util.*;

/**
 * The Options menu for JBother
 * @author Adam Olsen
 * @version 1.0
*/
class BuddyListOptionsMenu extends JMenu
{
	private ResourceBundle resources = ResourceBundle.getBundle( "JBotherBundle", Locale.getDefault() );
	private JCheckBoxMenuItem unfiledBuddies = new JCheckBoxMenuItem( resources.getString( "showUnfiledBuddies" ) );
	private JCheckBoxMenuItem agentBuddies = new JCheckBoxMenuItem( resources.getString( "showAgentBuddies" ) );
    private JCheckBoxMenuItem agentMessages = new JCheckBoxMenuItem( resources.getString( "recieveAgentMessages" ) );
	private JMenuItem priority = new JMenuItem( resources.getString( "setPriority" ) );
	private JMenuItem prefsItem = new JMenuItem( resources.getString( "preferences" ) );
	private JMenuItem pluginItem = new JMenuItem( resources.getString( "pluginManager" ) );
	private JMenuItem deleteAccount = new JMenuItem( resources.getString( "deleteAccount" ) );
	private JMenu managementMenu = new JMenu( resources.getString( "accountManagement" ) );
	private JMenuItem changePassword = new JMenuItem( resources.getString( "changePassword" ) );

	/**
	 * Creates the menu, adds the various items to it
	*/
	public BuddyListOptionsMenu()
	{
		super( "Options" );
		setText( resources.getString( "options" ) );

		MenuActionListener listener = new MenuActionListener();
		prefsItem.addActionListener( listener );

		unfiledBuddies.addActionListener( listener );
		agentBuddies.addActionListener( listener );
		pluginItem.addActionListener( listener );
        agentMessages.addActionListener( listener );
		priority.addActionListener( listener );

		//get the settings for the "show" states
		unfiledBuddies.setState( Settings.getInstance().getBoolean( "showUnfiledBuddies" ) );
		agentBuddies.setState( Settings.getInstance().getBoolean( "showAgentBuddies" ) );
        agentMessages.setState( Settings.getInstance().getBoolean( "showAgentMessages" ) );

		managementMenu.add( changePassword );
		managementMenu.add( deleteAccount );
		deleteAccount.addActionListener( listener );
		changePassword.addActionListener( listener );

		add( unfiledBuddies );
		add( agentBuddies );
        add( agentMessages );

		add( priority );

		if( Arguments.getInstance().getProperty( "webstart" ) == null )
		{
			add( pluginItem );
		}

		add( prefsItem );
		add( managementMenu );
	}

	/**
	 * @param var true if this is OS X
	*/
	public void setOSX( boolean var )
	{
		if( var )
		{
			remove( prefsItem );
		}
		else {
			if( !isMenuComponent( prefsItem ) ) insert( prefsItem, getItemCount() - 1 );
		}

	}

	/**
	 * Listens for an item to be clicked in the menu
	 * @author Adam Olsen
	 * @version 1.0
	*/
	class MenuActionListener implements ActionListener
	{
		public void actionPerformed( ActionEvent e )
		{
			if( e.getSource() == unfiledBuddies ) BuddyList.getInstance().getBuddyListTree().setShowUnfiledBuddies( unfiledBuddies.getState() );
			if( e.getSource() == agentBuddies ) BuddyList.getInstance().getBuddyListTree().setShowAgentBuddies( agentBuddies.getState() );
            if( e.getSource() == agentMessages ) BuddyList.getInstance().getBuddyListTree().setShowAgentMessages( agentMessages.getState() );
			if( e.getSource() == prefsItem ) if( !DialogTracker.containsDialog( PreferencesDialog.class )  ) new PreferencesDialog().show();
			if( e.getSource() == priority ) new PriorityDialog().show();
			if( e.getSource() == deleteAccount ) deleteAccountHandler();
			if( e.getSource() == changePassword )
				if( !DialogTracker.containsDialog( ChangePasswordDialog.class ) ) new ChangePasswordDialog().show();

			if( e.getSource() == pluginItem )
			{
				if( DialogTracker.containsDialog( PluginManager.class ) ) return;
				PluginManager manager = new PluginManager( "www.jbother.org", "/plugins/index.rb", JBother.settingsDir );
				manager.show();
			}
		}
	}

	/**
	 * Confirms that the user really wants to delete their account, and
	 * does so if they do
	*/
	private void deleteAccountHandler()
	{
		if( !BuddyList.getInstance().checkConnection() )
		{
			BuddyList.getInstance().connectionError();
			return;
		}

		int result = JOptionPane.showConfirmDialog( null,
				resources.getString( "sureDeleteAccount" ), resources.getString( "deleteAccount" ), JOptionPane.YES_NO_OPTION );

		// makes sure one more time :)
		if( result == 0 )
		{
			result = JOptionPane.showConfirmDialog( null,
				resources.getString( "sureDeleteAccountPart2" ), resources.getString( "deleteAccount" ), JOptionPane.YES_NO_OPTION );
		}

		if( result == 0 )
		{
			AccountManager manager = BuddyList.getInstance().getConnection().getAccountManager();

			// go ahead and delete the account
			try {
				manager.deleteAccount();
			}
			catch( XMPPException e )
			{
				String errorMessage = "";
				if( e.getXMPPError() != null && e.getXMPPError().getMessage() != null ) errorMessage = e.getXMPPError().getMessage();
				if( errorMessage == null && e.getMessage() != null ) errorMessage = e.getMessage();
				if( errorMessage == null ) errorMessage = resources.getString( "unknownError" );

				Standard.warningMessage( BuddyList.getInstance(), resources.getString( "deleteAccount" ), errorMessage );
				return;
			}

			// close everything and display a login dialog
			BuddyList.getInstance().kill();
			Standard.warningMessage( BuddyList.getInstance(), resources.getString( "deleteAccount" ),
				resources.getString( "accountDeleted" ) );
		}
	}
}
