/*
	Copyright (C) 2003 Adam Olsen

	This program is free software; you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation; either version 1, or (at your option)
	any later version.

	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with this program; if not, write to the Free Software
	Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/

package com.valhalla.jbother.menus;

import java.awt.*;
import java.awt.event.*;
import java.net.URL;
import java.util.*;

import javax.swing.*;

import org.jivesoftware.smack.*;
import org.jivesoftware.smack.packet.*;
import com.valhalla.gui.*;
import com.valhalla.jbother.*;
import com.valhalla.jbother.jabber.BuddyStatus;
import com.valhalla.settings.Settings;

/**
 * The menu that pops up when you right click on a user in your roster
 *
 * @author Adam Olsen
 * @version 1.0
*/
public class BuddyListPopupMenu extends JPopupMenu
{
	private ResourceBundle resources = ResourceBundle.getBundle( "JBotherBundle", Locale.getDefault() );
	private JMenuItem removeBuddyItem = new JMenuItem( resources.getString( "removeFromRoster" ) );
	private JMenuItem modifyBuddyItem = new JMenuItem( resources.getString( "modify" ) );
	private JMenu     resourceMenu = new JMenu( resources.getString( "resources" ) );
	private JMenu authMenu = new JMenu( resources.getString( "authorization" ) );
	private JMenuItem requestSubscription = new JMenuItem( resources.getString( "requestNotification" ) );
	private JMenuItem resendSubscription = new JMenuItem( resources.getString( "resendNotification" ) );
	private JMenuItem removeSubscription = new JMenuItem( resources.getString( "removeNotification" ) );

	private JMenuItem messageItem = new JMenuItem( resources.getString( "openChatDialog" ) );
	private JMenuItem logItem = new JMenuItem( resources.getString( "viewLog" ) );
	private JMenuItem infoItem = new JMenuItem( resources.getString( "getInfo" ) );
	private BuddyStatus buddy;
	private JTree tree;

	/**
	 * Creates the menu
	*/
	public BuddyListPopupMenu()
	{
		MenuActionListener listener = new MenuActionListener();

		infoItem.addActionListener( listener );
		removeBuddyItem.addActionListener( listener );
		messageItem.addActionListener( listener );
		modifyBuddyItem.addActionListener( listener );
		requestSubscription.addActionListener( listener );
		logItem.addActionListener( listener );
		resendSubscription.addActionListener( listener );
		removeSubscription.addActionListener( listener );

		authMenu.add( requestSubscription );
		authMenu.add( resendSubscription );
		authMenu.add( removeSubscription );

		add( messageItem );
		add( infoItem );
		add( logItem );
		add( removeBuddyItem );
		add( modifyBuddyItem );
		addSeparator();
		add( resourceMenu );
		add( authMenu );
	}

	/**
	 * Requests subscription to a user
	 * @param type the type of subscription to send
	*/
	private void subscriptionHandler( Presence.Type type )
	{
		Presence presence = new Presence( type );

		presence.setTo( buddy.getUser() );
		if( BuddyList.getInstance().checkConnection() )
			BuddyList.getInstance().getConnection().sendPacket( presence );
		else
			BuddyList.getInstance().connectionError();
	}

	/**
	 * Listens for different items to get clicked on in the popup menu
	 * @author Adam Olsen
	 * @version 1.0
	*/
	class MenuActionListener implements ActionListener
	{
		public void actionPerformed( ActionEvent e )
		{
			if( e.getSource() == removeBuddyItem ) removeBuddyHandler();
			else if( e.getSource() == messageItem ) BuddyList.getInstance().getBuddyListTree().initiateConversation();
			else if( e.getSource() == modifyBuddyItem ) modifyBuddyHandler();
			else if( e.getSource() == requestSubscription ) subscriptionHandler( Presence.Type.SUBSCRIBE );
			else if( e.getSource() == removeSubscription ) subscriptionHandler( Presence.Type.UNSUBSCRIBED );
			else if( e.getSource() == resendSubscription ) subscriptionHandler( Presence.Type.SUBSCRIBED );
			else if( e.getSource() == infoItem )
			{
				new InformationViewerDialog( buddy.getUser() + "/" + buddy.getHighestResource() ).show();
			}
			else if( e.getSource() == logItem )
			{
				new LogViewerDialog( null, buddy.getUser() );
			}
		}
	}

	/**
	 * Opens the AddBuddyDialog to modify a buddy
	*/
	public void modifyBuddyHandler()
	{
		AddBuddyDialog buddyDialog = new AddBuddyDialog();
		buddyDialog.setBuddy( buddy.getRosterEntry() );
		buddyDialog.show();
	}

	/**
	 * Confirms buddy removal, and then removes the buddy
	*/
	protected void removeBuddyHandler()
	{
		if( !BuddyList.getInstance().checkConnection() )
		{
			BuddyList.getInstance().connectionError();
			return;
		}

		int result = 1;

		result = JOptionPane.showConfirmDialog( null, resources.getString( "sureRemoveContact" ), resources.getString( "removeFromRoster" ),
			JOptionPane.YES_NO_OPTION );

		if( result != 0 ) return;

		RosterEntry entry = buddy.getRosterEntry();
		buddy.setRemoved( true );
		String gp = buddy.getGroup();

		if( entry != null )
		{
			com.valhalla.Logger.debug( "Remove entry user: " + entry.getUser() );
			BuddyList.getInstance().getBuddyListTree().removeBuddy( buddy, gp );

			Iterator iterator = entry.getGroups();
			while( iterator.hasNext() )
			{
				RosterGroup group = (RosterGroup)iterator.next();
				group.removeEntry( entry );
			}

			RosterPacket packet = new RosterPacket();
			packet.setType( IQ.Type.SET );
			RosterPacket.Item item = new RosterPacket.Item( entry.getUser(), entry.getName() );
			item.setItemType( RosterPacket.ItemType.REMOVE );
			packet.addRosterItem( item );

			BuddyList.getInstance().getConnection().sendPacket( packet );
		}
		else BuddyList.getInstance().getBuddyListTree().removeBuddy( buddy, gp );
	}

	/**
	 * Updates the resources menu
	*/
	public void updateResourceMenu()
	{
		ResourceActionListener listener = new ResourceActionListener( buddy );
		resourceMenu.removeAll();
		Set keys = buddy.keySet();

		Iterator i = keys.iterator();
		while( i.hasNext() )
		{
			boolean na = false;
			String key = (String)i.next();

			if( key.equals( "N/A" ) )
			{
				na = true;
				key = "None";
			}

			JMenuItem item = new JMenuItem( key );

			if( !na )
			{
				Presence.Mode mode;
				if( buddy.size() == 0 ) {
					mode = null;
					item.setForeground( Color.GRAY );
				} else {
					mode = buddy.getPresence( key );
				}

				ImageIcon icon = StatusIconCache.getStatusIcon(mode);
				if (icon != null) item.setIcon(icon);
				item.addActionListener( listener );
			}
			else item.setEnabled( false );

			resourceMenu.add( item );
		}
	}

	/**
	 * Shows the popup menu
	 * @param tree the tree to show the menu on
	 * @param x the x coordinate
	 * @param y the y coordinate
	 * @param buddy the buddy that this menu should show up on
	*/
	public void showMenu( Component tree, int x, int y, BuddyStatus buddy )
	{
		this.buddy = buddy;
		this.tree = (JTree)tree;

		updateResourceMenu();

		validate();

		show( tree, x, y );
	}

	/**
	 * Listens for a resource to get clicked in the ResourceMenu
	 */
	class ResourceActionListener implements ActionListener
	{
		private BuddyStatus buddy;

		public ResourceActionListener( BuddyStatus buddy )
		{
			this.buddy = buddy;
		}

		public void actionPerformed( ActionEvent e )
		{
			JMenuItem item = (JMenuItem)e.getSource();
			BuddyList.getInstance().getBuddyListTree().initiateConversation();
			ChatPanel window = (ChatPanel)buddy.getConversation();
			window.getResourceBox().setSelectedItem( item.getText() );
			window.getResourceBox().validate();
		}
	}
}
