/*
	Copyright (C) 2003 Adam Olsen

	This program is free software; you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation; either version 1, or (at your option)
	any later version.

	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with this program; if not, write to the Free Software
	Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/
package com.valhalla.jbother.menus;

import java.awt.*;
import java.awt.event.*;
import java.net.URL;
import java.util.*;

import javax.swing.*;

import org.jivesoftware.smack.*;
import org.jivesoftware.smack.packet.*;

import com.valhalla.gui.*;
import com.valhalla.jbother.*;
import com.valhalla.jbother.*;
import com.valhalla.jbother.jabber.BuddyStatus;
import com.valhalla.settings.Settings;

/**
 * The menu that pops up if someone right clicks on a user in a groupchat
 *
 * @author Adam Olsen
 * @version 1.0
*/
public class NickListPopupMenu extends JPopupMenu
{
	private ResourceBundle resources = ResourceBundle.getBundle( "JBotherBundle", Locale.getDefault() );
	private JMenuItem messageItem = new JMenuItem( resources.getString( "openChatDialog" ) );
	private JMenuItem logItem = new JMenuItem( resources.getString( "viewLog" ) );
	private JMenuItem infoItem = new JMenuItem( resources.getString( "getInfo" ) );
	private BuddyStatus buddy;

	/**
	 * Sets up the popup menu
	*/
	public NickListPopupMenu()
	{
		MenuActionListener listener = new MenuActionListener();

		infoItem.addActionListener( listener );
		messageItem.addActionListener( listener );
		logItem.addActionListener( listener );

		add( messageItem );
		add( infoItem );
		add( logItem );
	}

	/**
	 * Listens for a double mouse click, or a right click
	 * @author Adam Olsen
	 * @version 1.0
	*/
	class MenuActionListener implements ActionListener
	{
		public void actionPerformed( ActionEvent e )
		{
			if( e.getSource() == messageItem )
			{
				if( buddy.getConversation() == null )
				{
					ChatPanel conver = new ChatPanel( buddy );
					conver.setVisible( true );
					buddy.setConversation( conver );
					MessageDelegator.getInstance().showPanel( buddy.getConversation() );
					MessageDelegator.getInstance().frontFrame( buddy.getConversation() );
				}
				else
				{
					MessageDelegator.getInstance().showPanel( buddy.getConversation() );
					MessageDelegator.getInstance().frontFrame( buddy.getConversation() );

				}
			}
			else if( e.getSource() == infoItem )
			{
				new InformationViewerDialog( buddy.getUser() ).show();
			}
			else if( e.getSource() == logItem )
			{
				new LogViewerDialog( null, buddy.getUser() );
			}
		}
	}

	/**
	 * Displays the popup menu
	 * @param comp the component to pop the menu up on
	 * @param x the x coordinate
	 * @param y the y coordinate of the menu
	 * @param buddy the BuddyStatus that was clicked on
	*/
	public void showMenu( Component comp, int x, int y, BuddyStatus buddy )
	{
		this.buddy = buddy;
		validate();
		show( comp, x, y );
	}
}
