/*
    Copyright (C) 2003 Adam Olsen

	This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 1, or (at your option)
    any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/

package com.valhalla.jbother.preferences;

import java.awt.*;
import javax.swing.*;
import java.util.*;
import com.valhalla.jbother.*;
import com.valhalla.jbother.jabber.*;

import com.valhalla.gui.*;
import com.valhalla.settings.Settings;
import com.valhalla.settings.TempSettings;

/**
 * Allows the user to change General Preferences
 * @author Adam Olsen
 * @version 1.0
*/
class GeneralPreferencesPanel extends JPanel implements PreferencesPanel
{
	private ResourceBundle resources = ResourceBundle.getBundle( "JBotherBundle", Locale.getDefault() );
	private PreferencesDialog prefs;
	private GridBagLayout grid = new GridBagLayout();
	private GridBagConstraints c = new GridBagConstraints();
	private JCheckBox focusWindow = new JCheckBox( resources.getString( "focusConversations" ) );
	private JCheckBox keepLogs = new JCheckBox( resources.getString( "keepLogs" ) );
	private JCheckBox preserve = new JCheckBox( resources.getString( "preserveClosed" ) );
	private JCheckBox autoAway = new JCheckBox( resources.getString( "setAutoAway" ) );
	private JCheckBox nonICQ = new JCheckBox( resources.getString( "useNonICQ" ) );
	private JCheckBox noSplash = new JCheckBox( resources.getString( "noSplash" ) );
	private JTextField displayName = new JTextField( 15 );
	private JLabel displayLabel = new JLabel( resources.getString( "displayedName" ) );

	/**
	 * Creates the PreferencesPanel
	 * @param dialog the enclosing PreferencesDialog
	*/
	public GeneralPreferencesPanel( PreferencesDialog dialog )
	{
		this.prefs = dialog;
		setBorder( BorderFactory.createTitledBorder( resources.getString( "generalPreferences" ) ) );
		setLayout( grid );

		c.gridx = 0;
		c.gridy = 0;
		c.fill = GridBagConstraints.HORIZONTAL;
		c.anchor = GridBagConstraints.WEST;
		c.weightx = .5;
		c.gridwidth = 2;
		grid.setConstraints( focusWindow, c );
		add( focusWindow );

		c.gridy++;
		grid.setConstraints( keepLogs, c );
		add( keepLogs );

		c.gridy++;
		grid.setConstraints( preserve, c );
		add( preserve );

		c.gridy++;
		grid.setConstraints( autoAway, c );
		add( autoAway );

		c.gridy++;
		grid.setConstraints( nonICQ, c );
		add( nonICQ );

		//c.gridy++;
		//grid.setConstraints( noSplash, c );
		//add( noSplash );

		//display name stuff
		displayLabel.setBorder( BorderFactory.createEmptyBorder( 0, 5, 0, 5 ) );
		c.gridwidth = 1;
		c.gridy++;
		c.weightx = 0.0;
		c.fill = GridBagConstraints.NONE;
		grid.setConstraints( displayLabel, c );
		add( displayLabel );
		c.gridx = 1;
		c.weightx = 1.0;

		grid.setConstraints( displayName, c );
		add( displayName );


		//this is the space taker
		JLabel blankLabel = new JLabel( "" );
		c.weighty = .5;
		c.gridx = 0;
		c.gridwidth = 2;
		c.gridy++;
		grid.setConstraints( blankLabel, c );
		add( blankLabel );

		loadSettings();
	}

	/**
	 * Loads settings from the settings file and fills out the form for defaults
	*/
	private void loadSettings()
	{
		displayName.setText( Settings.getInstance().getProperty( "myDisplayedName" ) );
		focusWindow.setSelected( Settings.getInstance().getBoolean( "focusWindow" ) );
		keepLogs.setSelected( Settings.getInstance().getBoolean( "keepLogs" ) );
		preserve.setSelected( Settings.getInstance().getBoolean( "preserveMessages" ) );
		autoAway.setSelected( Settings.getInstance().getBoolean( "autoAway" ) );
		noSplash.setSelected( Settings.getInstance().getBoolean( "noSplash" ) );
		nonICQ.setSelected( Settings.getInstance().getBoolean( "useTabbedWindow" ) );
	}

	/**
	 * Switches the application to a non-ICQ style interface
	*/
	public void switchToNonICQInterface()
	{
		com.valhalla.Logger.debug( "Switching to non-ICQ style interface" );

		Vector panels = MessageDelegator.getInstance().getPanels();
		Iterator i = panels.iterator();
		while( i.hasNext() )
		{
			ConversationPanel panel = (ConversationPanel)i.next();

			if( panel instanceof MessagePanel ) continue;
			JFrame frame = panel.getContainingFrame();

			if( frame == null ) continue;
			boolean visible = frame.isVisible();
			frame.dispose();

			panel.setContainingFrame( null );
			if( visible ) BuddyList.getInstance().addTabPanel( panel );
		}
	}

	/**
	 * Switches the application to an ICQ style interface
	*/
	public void switchToICQInterface()
	{
		com.valhalla.Logger.debug( "Switching to ICQ style interface..." );

		Vector panels = MessageDelegator.getInstance().getPanels();
		Iterator i = panels.iterator();
		while( i.hasNext() )
		{
			ConversationPanel panel = (ConversationPanel)i.next();
			if( panel instanceof MessagePanel ) continue;

			if( BuddyList.getInstance().getTabFrame() != null &&
				BuddyList.getInstance().getTabFrame().contains( panel ) )
			{
				BuddyList.getInstance().removeTabPanel( panel );
				panel.createFrame();
			}
		}
	}

	/**
	 * Returns the currently chosen settings
	*/
	public TempSettings getSettings()
	{
		TempSettings mySettings = new TempSettings();

		mySettings.setBoolean( "focusWindow", focusWindow.isSelected() );
		mySettings.setBoolean( "keepLogs", keepLogs.isSelected() );
		mySettings.setBoolean( "preserveMessages", preserve.isSelected() );
		mySettings.setProperty( "myDisplayedName", displayName.getText() );
		mySettings.setBoolean( "autoAway", autoAway.isSelected() );
		mySettings.setBoolean( "noSplash", noSplash.isSelected() );
		mySettings.setBoolean( "useTabbedWindow", nonICQ.isSelected() );

		if( !Settings.getInstance().getBoolean( "useTabbedWindow" ) && mySettings.getBoolean( "useTabbedWindow" ) )
		{
			switchToNonICQInterface();
		}

		if( Settings.getInstance().getBoolean( "useTabbedWindow" ) && !mySettings.getBoolean( "useTabbedWindow" ) )
		{
			switchToICQInterface();
		}

		return mySettings;
	}
}
