/*
    Copyright (C) 2003 Adam Olsen

	This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 1, or (at your option)
    any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/

package com.valhalla.jbother.preferences;

import java.awt.Dimension;
import java.awt.event.MouseAdapter;
import java.awt.event.MouseEvent;

import java.util.*;
import com.valhalla.jbother.*;

import javax.swing.*;
import javax.swing.tree.*;

/**
 * Displays the different preference panels available in a JTree
 * @author Adam Olsen
 * @version 1.0
*/
public class PreferencesTree extends JPanel
{
	private ResourceBundle resources = ResourceBundle.getBundle( "JBotherBundle", Locale.getDefault() );
	private PreferencesDialog prefsDialog;
	private JTree tree;
	private DefaultMutableTreeNode root;
	private Properties panels = new Properties();

	/**
	 * Sets up the preferences tree
	 * @param prefsDialog the enclosing preferences dialog
	*/
	public PreferencesTree( PreferencesDialog prefsDialog )
	{
		this.prefsDialog = prefsDialog;

		panels.setProperty( resources.getString( "general" ), "General" );
		panels.setProperty( resources.getString( "applications" ), "Applications" );
		panels.setProperty( resources.getString( "sounds" ), "Sounds" );
		panels.setProperty( resources.getString( "appearance" ), "Appearance" );
		panels.setProperty( resources.getString( "privacy" ), "Privacy" );

		setupTree();


		setLayout( new BoxLayout( this, BoxLayout.Y_AXIS ) );
		add( new JScrollPane( tree ) );
		setPreferredSize( new Dimension( 150, 350 ) );
	}

	/**
	 * Sets the tree up
	*/
	private void setupTree()
	{
		root = new DefaultMutableTreeNode( resources.getString( "preferences" ) );

		DefaultTreeCellRenderer renderer = new DefaultTreeCellRenderer();
		renderer.setLeafIcon( null );

		String items[] = new String[] {
			resources.getString( "general" ),
			resources.getString( "applications" ),
			resources.getString( "sounds" ),
			resources.getString( "appearance" ),
			resources.getString( "privacy" ) };


		for( int i = 0; i < items.length; i++ )
		{
			root.add( new DefaultMutableTreeNode( items[i] ) );
		}

		tree = new JTree( root );
		tree.setShowsRootHandles( true );
		tree.setSelectionRow( 1 );
		tree.setCellRenderer( renderer );
		tree.addMouseListener( new DoubleClickListener() );
	}

	/**
	 * Gets the index of the selected item
	 * @return the selected index
	*/
	public int getSelectedIndex()
	{
		TreePath path = tree.getSelectionPath();
		DefaultMutableTreeNode node = (DefaultMutableTreeNode)path.getLastPathComponent();
		DefaultTreeModel model = (DefaultTreeModel)tree.getModel();
		return model.getIndexOfChild( root, node );
	}

	/**
	 * Gets the number of rows
	 * @return the row count
	*/
	public int getRowCount()
	{
		DefaultTreeModel model = (DefaultTreeModel)tree.getModel();
		return model.getChildCount( root );
	}

	/**
	 * Listens for a click on one of the tree items
	*/
	class DoubleClickListener extends MouseAdapter
	{
		public void mouseClicked( MouseEvent e )
		{
			JTree tree = (JTree)e.getComponent();
			TreePath path = tree.getSelectionPath();
			DefaultMutableTreeNode node = (DefaultMutableTreeNode)path.getLastPathComponent();
			String string = panels.getProperty( (String)node.getUserObject() );
			if( string != null ) prefsDialog.switchPanel( string );
		}
	}
}
