# Prints a list of all available plugins for a specified APIVersion
def pluginList
	puts "Plugin list will follow"
	puts "fileName\tname\tdescription\tAPIVersion\tversion\tsize\tauthor\treleaseDate\tos\tarch"

	# get a list of all the jar files in the plugin dir and print info for them
	Dir[$pluginDir + "*.jar"].each do |jar|

		# get the plugin information hash
		info = getPluginInfo( jar )

		print jar.sub( /.*\//, '' ) + "\t"

		# fill in the possibly empty spots
		info['author'] = "Unknown" if info['author'].nil?
		info['releaseDate'] = "Unknown" if info['releaseDate'].nil?
		info['os'] = "all" if info['os'].nil?
		info['arch'] = "all" if info['arch'].nil?

		array = Array.[]( 'name', 'description', 'APIVersion', 'version', 'size', 'author', 'releaseDate', 'os', 'arch' )
		string = String.new

		array.each do |item|
			string += info[item] + "\t"
		end

		# cut of the last uneeded tab
		string.chop!
		puts string
	end
end

# read a plugin and print it out
def getPlugin
	throwErr "no plugin was specified" if $plugin.empty?

	file = File.new( $pluginDir + $plugin, "r" )

	while !file.eof?
		print file.read( 1024 )
	end

	file.close
end

# find the plugin.properties file in the zip file and parse it
# into name value pairs
def getPluginInfo( jar )
	Zip::ZipInputStream.open( jar ) do |z|
		while entry = z.get_next_entry
			if entry.name == "plugin.properties" then
				data = String.new
				while ( line = z.gets ) != nil
					data += line
				end

				info = parseInfo( data )

				info['size'] = File.size( jar ).to_s

				return info
			end
		end
	end
end

# parse a block of text into name value pairs
def parseInfo( data )
	info = Hash.new
	array = data.split( /\n/ )
	array.each do |line|
		( key, value ) = line.split( /\s?=\s?/ )
		info[key] = value
	end

	return info
end

def throwErr( message )
	print "Error: " + message
	exit
end
